"""Generated message classes for biglake version v1beta.

The BigLake API provides access to BigLake Metastore, a serverless, fully
managed, and highly available metastore for open-source data that can be used
for querying Apache Iceberg tables in BigQuery.
"""
# NOTE: This file is autogenerated and should not be edited by hand.

from apitools.base.protorpclite import messages as _messages
from apitools.base.py import encoding


package = 'biglake'


class BatchCreatePartitionsRequest(_messages.Message):
  r"""Request message for the BatchCreatePartitions method.

  Fields:
    requests: Required. Requests to add partitions to the table.
    skipExistingPartitions: Optional. Corresponds to the `ifNotExists` flag in
      the Hive Metastore APIs. If the flag is set to false, the server will
      return ALREADY_EXISTS if any partition already exists. If the flag is
      set to true, the server will skip existing partitions and insert only
      the non-existing partitions. A maximum of 900 partitions can be inserted
      in a batch.
  """

  requests = _messages.MessageField('CreatePartitionRequest', 1, repeated=True)
  skipExistingPartitions = _messages.BooleanField(2)


class BatchCreatePartitionsResponse(_messages.Message):
  r"""Response message for BatchCreatePartitions.

  Fields:
    partitions: The list of partitions that have been added.
  """

  partitions = _messages.MessageField('Partition', 1, repeated=True)


class BatchDeletePartitionsRequest(_messages.Message):
  r"""Request message for BatchDeletePartitions. The Partition is uniquely
  identified by values, which is an ordered list. Hence, there is no separate
  name or partition id field.

  Fields:
    partitionValues: Required. The list of partitions (identified by its
      values) to be deleted. A maximum of 900 partitions can be deleted in a
      batch.
  """

  partitionValues = _messages.MessageField('PartitionValues', 1, repeated=True)


class BatchUpdatePartitionsRequest(_messages.Message):
  r"""Request message for BatchUpdatePartitions.

  Fields:
    requests: Required. Requests to update partitions in the table.
  """

  requests = _messages.MessageField('UpdatePartitionRequest', 1, repeated=True)


class BatchUpdatePartitionsResponse(_messages.Message):
  r"""Response message for BatchUpdatePartitions.

  Fields:
    partitions: The list of partitions that have been updated. A maximum of
      900 partitions can be updated in a batch.
  """

  partitions = _messages.MessageField('Partition', 1, repeated=True)


class BiglakeHiveV1betaProjectsCatalogsCreateRequest(_messages.Message):
  r"""A BiglakeHiveV1betaProjectsCatalogsCreateRequest object.

  Fields:
    hiveCatalog: A HiveCatalog resource to be passed as the request body.
    hiveCatalogId: Required. The Hive Catalog ID to use for the catalog that
      will become the final component of the catalog's resource name. The
      maximum length is 256 characters.
    parent: Required. The parent resource where this catalog will be created.
      Format: projects/{project_id_or_number}
    primary_location: Required. The GCP region that specifies where the
      catalog metadata is stored, e.g. us-central1, EU, etc.
  """

  hiveCatalog = _messages.MessageField('HiveCatalog', 1)
  hiveCatalogId = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)
  primary_location = _messages.StringField(4)


class BiglakeHiveV1betaProjectsCatalogsDatabasesCreateRequest(_messages.Message):
  r"""A BiglakeHiveV1betaProjectsCatalogsDatabasesCreateRequest object.

  Fields:
    hiveDatabase: A HiveDatabase resource to be passed as the request body.
    hiveDatabaseId: Required. The ID to use for the Hive Database. The maximum
      length is 128 characters.
    parent: Required. The parent resource where this database will be created.
      Format: projects/{project_id_or_number}/catalogs/{catalog_id}
  """

  hiveDatabase = _messages.MessageField('HiveDatabase', 1)
  hiveDatabaseId = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class BiglakeHiveV1betaProjectsCatalogsDatabasesDeleteRequest(_messages.Message):
  r"""A BiglakeHiveV1betaProjectsCatalogsDatabasesDeleteRequest object.

  Fields:
    name: Required. The name of the database to delete. Format: projects/{proj
      ect_id_or_number}/catalogs/{catalog_id}/databases/{database_id}
  """

  name = _messages.StringField(1, required=True)


class BiglakeHiveV1betaProjectsCatalogsDatabasesGetRequest(_messages.Message):
  r"""A BiglakeHiveV1betaProjectsCatalogsDatabasesGetRequest object.

  Fields:
    name: Required. The name of the database to retrieve. Format: projects/{pr
      oject_id_or_number}/catalogs/{catalog_id}/databases/{database_id}
  """

  name = _messages.StringField(1, required=True)


class BiglakeHiveV1betaProjectsCatalogsDatabasesListRequest(_messages.Message):
  r"""A BiglakeHiveV1betaProjectsCatalogsDatabasesListRequest object.

  Fields:
    pageSize: Optional. Page size for pagination.
    pageToken: Optional. PageToken for pagination.
    parent: Required. The hive catalog to list databases from. Format:
      projects/{project_id_or_number}/catalogs/{catalog_id}
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class BiglakeHiveV1betaProjectsCatalogsDatabasesPatchRequest(_messages.Message):
  r"""A BiglakeHiveV1betaProjectsCatalogsDatabasesPatchRequest object.

  Fields:
    hiveDatabase: A HiveDatabase resource to be passed as the request body.
    name: Identifier. The resource name. Format: projects/{project_id_or_numbe
      r}/catalogs/{catalog_id}/databases/{database_id}
    updateMask: Optional. The list of fields to update.
  """

  hiveDatabase = _messages.MessageField('HiveDatabase', 1)
  name = _messages.StringField(2, required=True)
  updateMask = _messages.StringField(3)


class BiglakeHiveV1betaProjectsCatalogsDatabasesTablesCreateRequest(_messages.Message):
  r"""A BiglakeHiveV1betaProjectsCatalogsDatabasesTablesCreateRequest object.

  Fields:
    hiveTable: A HiveTable resource to be passed as the request body.
    hiveTableId: Required. The Hive Table ID to use for the table that will
      become the final component of the table's resource name. The maximum
      length is 256 characters.
    parent: Required. The parent resource for the table to be created. Format:
      projects/{project_id_or_number}/catalogs/{catalog_id}/databases/{databas
      e_id}
  """

  hiveTable = _messages.MessageField('HiveTable', 1)
  hiveTableId = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class BiglakeHiveV1betaProjectsCatalogsDatabasesTablesDeleteRequest(_messages.Message):
  r"""A BiglakeHiveV1betaProjectsCatalogsDatabasesTablesDeleteRequest object.

  Fields:
    name: Required. The name of the database to delete. Format: projects/{proj
      ect_id_or_number}/catalogs/{catalog_id}/databases/{database_id}/tables/{
      table_id}
  """

  name = _messages.StringField(1, required=True)


class BiglakeHiveV1betaProjectsCatalogsDatabasesTablesGetRequest(_messages.Message):
  r"""A BiglakeHiveV1betaProjectsCatalogsDatabasesTablesGetRequest object.

  Fields:
    name: Required. The name of the table to retrieve. Format: projects/{proje
      ct_id_or_number}/catalogs/{catalog_id}/databases/{database_id}/tables/{t
      able_id}
  """

  name = _messages.StringField(1, required=True)


class BiglakeHiveV1betaProjectsCatalogsDatabasesTablesListRequest(_messages.Message):
  r"""A BiglakeHiveV1betaProjectsCatalogsDatabasesTablesListRequest object.

  Fields:
    pageSize: Optional. Page size for pagination.
    pageToken: Optional. PageToken for pagination.
    parent: Required. The database to list tables from. Format: projects/{proj
      ect_id_or_number}/catalogs/{catalog_id}/databases/{database_id}
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class BiglakeHiveV1betaProjectsCatalogsDatabasesTablesPartitionsBatchCreateRequest(_messages.Message):
  r"""A
  BiglakeHiveV1betaProjectsCatalogsDatabasesTablesPartitionsBatchCreateRequest
  object.

  Fields:
    batchCreatePartitionsRequest: A BatchCreatePartitionsRequest resource to
      be passed as the request body.
    parent: Required. Reference to the table to where the partitions to be
      added, in the format of projects/{project}/catalogs/{catalogs}/databases
      /{database}/tables/{table}.
  """

  batchCreatePartitionsRequest = _messages.MessageField('BatchCreatePartitionsRequest', 1)
  parent = _messages.StringField(2, required=True)


class BiglakeHiveV1betaProjectsCatalogsDatabasesTablesPartitionsBatchDeleteRequest(_messages.Message):
  r"""A
  BiglakeHiveV1betaProjectsCatalogsDatabasesTablesPartitionsBatchDeleteRequest
  object.

  Fields:
    batchDeletePartitionsRequest: A BatchDeletePartitionsRequest resource to
      be passed as the request body.
    parent: Required. Reference to the table to which these partitions belong,
      in the format of projects/{project}/catalogs/{catalogs}/databases/{datab
      ase}/tables/{table}.
  """

  batchDeletePartitionsRequest = _messages.MessageField('BatchDeletePartitionsRequest', 1)
  parent = _messages.StringField(2, required=True)


class BiglakeHiveV1betaProjectsCatalogsDatabasesTablesPartitionsBatchUpdateRequest(_messages.Message):
  r"""A
  BiglakeHiveV1betaProjectsCatalogsDatabasesTablesPartitionsBatchUpdateRequest
  object.

  Fields:
    batchUpdatePartitionsRequest: A BatchUpdatePartitionsRequest resource to
      be passed as the request body.
    parent: Required. Reference to the table to which these partitions belong,
      in the format of projects/{project}/catalogs/{catalogs}/databases/{datab
      ase}/tables/{table}.
  """

  batchUpdatePartitionsRequest = _messages.MessageField('BatchUpdatePartitionsRequest', 1)
  parent = _messages.StringField(2, required=True)


class BiglakeHiveV1betaProjectsCatalogsDatabasesTablesPartitionsListRequest(_messages.Message):
  r"""A BiglakeHiveV1betaProjectsCatalogsDatabasesTablesPartitionsListRequest
  object.

  Fields:
    filter: Optional. SQL text filtering statement, similar to a `WHERE`
      clause in a query. Only supports single-row expressions. Aggregate
      functions are not supported. Examples: * `"int_field > 5"` *
      `"date_field = CAST('2014-9-27' as DATE)"` * `"nullable_field is not
      NULL"` * `"st_equals(geo_field, st_geofromtext("POINT(2, 2)"))"` *
      `"numeric_field BETWEEN 1.0 AND 5.0"` Restricted to a maximum length of
      1 MB.
    parent: Required. Reference to the table to which these partitions belong,
      in the format of projects/{project}/catalogs/{catalogs}/databases/{datab
      ase}/tables/{table}.
  """

  filter = _messages.StringField(1)
  parent = _messages.StringField(2, required=True)


class BiglakeHiveV1betaProjectsCatalogsDatabasesTablesPatchRequest(_messages.Message):
  r"""A BiglakeHiveV1betaProjectsCatalogsDatabasesTablesPatchRequest object.

  Fields:
    hiveTable: A HiveTable resource to be passed as the request body.
    name: Identifier. The resource name. Format: projects/{project_id_or_numbe
      r}/catalogs/{catalog_id}/databases/{database_id}/tables/{table_id}
    updateMask: Optional. The list of fields to update.
  """

  hiveTable = _messages.MessageField('HiveTable', 1)
  name = _messages.StringField(2, required=True)
  updateMask = _messages.StringField(3)


class BiglakeHiveV1betaProjectsCatalogsDeleteRequest(_messages.Message):
  r"""A BiglakeHiveV1betaProjectsCatalogsDeleteRequest object.

  Fields:
    name: Required. The name of the catalog to delete. Format:
      projects/{project_id_or_number}/catalogs/{catalog_id}
  """

  name = _messages.StringField(1, required=True)


class BiglakeHiveV1betaProjectsCatalogsGetRequest(_messages.Message):
  r"""A BiglakeHiveV1betaProjectsCatalogsGetRequest object.

  Fields:
    name: Required. The name of the catalog to retrieve. Format:
      projects/{project_id_or_number}/catalogs/{catalog_id}
  """

  name = _messages.StringField(1, required=True)


class BiglakeHiveV1betaProjectsCatalogsListRequest(_messages.Message):
  r"""A BiglakeHiveV1betaProjectsCatalogsListRequest object.

  Fields:
    pageSize: Optional. Page size for pagination.
    pageToken: Optional. Page token for pagination.
    parent: Required. The project to list catalogs from. Format:
      projects/{project_id_or_number}
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class BiglakeHiveV1betaProjectsCatalogsPatchRequest(_messages.Message):
  r"""A BiglakeHiveV1betaProjectsCatalogsPatchRequest object.

  Fields:
    hiveCatalog: A HiveCatalog resource to be passed as the request body.
    name: Identifier. The resource name. Format:
      projects/{project_id_or_number}/catalogs/{catalog_id}
    updateMask: Optional. The list of fields to update. For the `FieldMask`
      definition, see https://developers.google.com/protocol-
      buffers/docs/reference/google.protobuf#fieldmask If not set, defaults to
      all of the fields that are allowed to update.
  """

  hiveCatalog = _messages.MessageField('HiveCatalog', 1)
  name = _messages.StringField(2, required=True)
  updateMask = _messages.StringField(3)


class CreatePartitionRequest(_messages.Message):
  r"""Request message for CreatePartition. The Partition is uniquely
  identified by values, which is an ordered list. Hence, there is no separate
  name or partition id field.

  Fields:
    parent: Required. Reference to the table to where the partition to be
      added, in the format of projects/{project}/catalogs/{catalogs}/databases
      /{databases}/tables/{table}.
    partition: Required. The partition to be added.
  """

  parent = _messages.StringField(1)
  partition = _messages.MessageField('Partition', 2)


class Empty(_messages.Message):
  r"""A generic empty message that you can re-use to avoid defining duplicated
  empty messages in your APIs. A typical example is to use it as the request
  or the response type of an API method. For instance: service Foo { rpc
  Bar(google.protobuf.Empty) returns (google.protobuf.Empty); }
  """



class FieldSchema(_messages.Message):
  r"""Field schema information.

  Fields:
    comment: Optional. Comment of the field. The maximum length is 256
      characters.
    name: Required. Name of the field. The maximum length is 767 characters.
    type: Required. Type of the field. The maximum length is 128 characters.
  """

  comment = _messages.StringField(1)
  name = _messages.StringField(2)
  type = _messages.StringField(3)


class HiveCatalog(_messages.Message):
  r"""The HiveCatalog contains spark/hive databases and tables in the BigLake
  Metastore. While creating resources under a catalog, ideally ensure that the
  storage bucket location, spark / hive engine location or any other compute
  location match. Catalog can be viewed as the destination for migrating an
  on-prem Hive metastore to GCP.

  Fields:
    createTime: Output only. The creation time of the catalog.
    description: Optional. Stores the catalog description. The maximum length
      is 4000 characters.
    locationUri: Required. The Cloud Storage location path where the catalog
      exists. Format: gs://bucket/path/to/catalog The maximum length is 4000
      characters.
    name: Identifier. The resource name. Format:
      projects/{project_id_or_number}/catalogs/{catalog_id}
    replicas: Output only. The replicas for the catalog metadata.
    updateTime: Output only. The update time of the catalog.
  """

  createTime = _messages.StringField(1)
  description = _messages.StringField(2)
  locationUri = _messages.StringField(3)
  name = _messages.StringField(4)
  replicas = _messages.MessageField('Replica', 5, repeated=True)
  updateTime = _messages.StringField(6)


class HiveDatabase(_messages.Message):
  r"""Stores the hive database information. It includes the database name,
  description, location and properties associated with the database.

  Messages:
    ParametersValue: Optional. Stores the properties associated with the
      database. The maximum size is 2 MiB.

  Fields:
    createTime: Output only. The creation time of the database.
    description: Optional. Stores the database description. The maximum length
      is 4000 characters.
    locationUri: Optional. The Cloud Storage location path where the database
      exists. Format: `gs://bucket/path/to/database` If unspecified, the
      database will be stored in the catalog location. The maximum length is
      4000 characters.
    name: Identifier. The resource name. Format: projects/{project_id_or_numbe
      r}/catalogs/{catalog_id}/databases/{database_id}
    parameters: Optional. Stores the properties associated with the database.
      The maximum size is 2 MiB.
    updateTime: Output only. The update time of the database.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ParametersValue(_messages.Message):
    r"""Optional. Stores the properties associated with the database. The
    maximum size is 2 MiB.

    Messages:
      AdditionalProperty: An additional property for a ParametersValue object.

    Fields:
      additionalProperties: Additional properties of type ParametersValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ParametersValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  createTime = _messages.StringField(1)
  description = _messages.StringField(2)
  locationUri = _messages.StringField(3)
  name = _messages.StringField(4)
  parameters = _messages.MessageField('ParametersValue', 5)
  updateTime = _messages.StringField(6)


class HiveTable(_messages.Message):
  r"""Stores the hive table information. It includes the table name, schema
  (column names and types), data location, storage format, serde info, etc.
  This message closely matches the Table object in the IMetastoreClient

  Messages:
    ParametersValue: Optional. Stores the properties associated with the
      table. The maximum size is 4MiB.

  Fields:
    createTime: Output only. The creation time of the table.
    description: Optional. Description of the table. The maximum length is
      4000 characters.
    name: Identifier. The resource name. Format: projects/{project_id_or_numbe
      r}/catalogs/{catalog_id}/databases/{database_id}/tables/{table_id}
    parameters: Optional. Stores the properties associated with the table. The
      maximum size is 4MiB.
    partitionKeys: Optional. The partition keys of the table.
    storageDescriptor: Required. Storage descriptor of the table.
    tableType: Output only. The type of the table. This is EXTERNAL for
      BigLake hive tables.
    updateTime: Output only. The update time of the table.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ParametersValue(_messages.Message):
    r"""Optional. Stores the properties associated with the table. The maximum
    size is 4MiB.

    Messages:
      AdditionalProperty: An additional property for a ParametersValue object.

    Fields:
      additionalProperties: Additional properties of type ParametersValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ParametersValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  createTime = _messages.StringField(1)
  description = _messages.StringField(2)
  name = _messages.StringField(3)
  parameters = _messages.MessageField('ParametersValue', 4)
  partitionKeys = _messages.MessageField('FieldSchema', 5, repeated=True)
  storageDescriptor = _messages.MessageField('StorageDescriptor', 6)
  tableType = _messages.StringField(7)
  updateTime = _messages.StringField(8)


class ListHiveCatalogsResponse(_messages.Message):
  r"""Response message for the ListHiveCatalogs method.

  Fields:
    catalogs: Output only. The catalogs from the specified project.
    nextPageToken: Output only. A token, which can be sent as `page_token` to
      retrieve the next page. If this field is omitted, there are no
      subsequent pages.
    unreachable: Output only. The list of unreachable cloud regions. If non-
      empty, the result set might be incomplete.
  """

  catalogs = _messages.MessageField('HiveCatalog', 1, repeated=True)
  nextPageToken = _messages.StringField(2)
  unreachable = _messages.StringField(3, repeated=True)


class ListHiveDatabasesResponse(_messages.Message):
  r"""Response message for the ListHiveDatabases method.

  Fields:
    databases: Output only. The databases from the specified project and
      catalog.
    nextPageToken: Output only. A token, which can be sent as `page_token` to
      retrieve the next page. If this field is omitted, there are no
      subsequent pages.
  """

  databases = _messages.MessageField('HiveDatabase', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class ListHiveTablesResponse(_messages.Message):
  r"""Response message for the ListHiveTables method.

  Fields:
    nextPageToken: Output only. A token, which can be sent as `page_token` to
      retrieve the next page. If this field is omitted, there are no
      subsequent pages.
    tables: Output only. The tables from the specified project, catalog and
      database.
  """

  nextPageToken = _messages.StringField(1)
  tables = _messages.MessageField('HiveTable', 2, repeated=True)


class ListPartitionsResponse(_messages.Message):
  r"""Response message for ListPartitions.

  Fields:
    partitions: Output only. List of partitions.
  """

  partitions = _messages.MessageField('Partition', 1, repeated=True)


class Order(_messages.Message):
  r"""Sort order of the stored data per column.

  Fields:
    col: Required. The column name. The maximum length is 767 characters.
    order: Required. Defines the sort order of the column. Ascending if 1,
      descending if 0.
  """

  col = _messages.StringField(1)
  order = _messages.IntegerField(2, variant=_messages.Variant.INT32)


class Partition(_messages.Message):
  r"""Information about a Hive partition.

  Messages:
    ParametersValue: Optional. Additional parameters or metadata associated
      with the partition. Maximum size 10 KiB.

  Fields:
    createTime: Output only. The creation time of the partition.
    fields: Optional. List of columns.
    parameters: Optional. Additional parameters or metadata associated with
      the partition. Maximum size 10 KiB.
    storageDescriptor: Optional. Contains information about the physical
      storage of the data in the partition.
    values: Required. Represents the values of the partition keys, where each
      value corresponds to a specific partition key in the order in which the
      keys are defined. Each value is limited to 1024 characters.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ParametersValue(_messages.Message):
    r"""Optional. Additional parameters or metadata associated with the
    partition. Maximum size 10 KiB.

    Messages:
      AdditionalProperty: An additional property for a ParametersValue object.

    Fields:
      additionalProperties: Additional properties of type ParametersValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ParametersValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  createTime = _messages.StringField(1)
  fields = _messages.MessageField('FieldSchema', 2, repeated=True)
  parameters = _messages.MessageField('ParametersValue', 3)
  storageDescriptor = _messages.MessageField('StorageDescriptor', 4)
  values = _messages.StringField(5, repeated=True)


class PartitionValues(_messages.Message):
  r"""Represents the values of a partition.

  Fields:
    values: Required. The values of the partition keys, where each value
      corresponds to a specific partition key in the order in which the keys
      are defined.
  """

  values = _messages.StringField(1, repeated=True)


class Replica(_messages.Message):
  r"""The replica of the Catalog.

  Enums:
    StateValueValuesEnum: Output only. The current state of the replica.

  Fields:
    region: Output only. The region of the replica. For example `us-east1`.
    state: Output only. The current state of the replica.
  """

  class StateValueValuesEnum(_messages.Enum):
    r"""Output only. The current state of the replica.

    Values:
      STATE_UNSPECIFIED: The replica state is unknown.
      STATE_PRIMARY: Indicates the replica is the writable primary.
      STATE_PRIMARY_IN_PROGRESS: Indicates the replica has been recently
        assigned as the primary, but not all databases are writeable yet.
      STATE_SECONDARY: Indicates the replica is a read-only secondary replica.
    """
    STATE_UNSPECIFIED = 0
    STATE_PRIMARY = 1
    STATE_PRIMARY_IN_PROGRESS = 2
    STATE_SECONDARY = 3

  region = _messages.StringField(1)
  state = _messages.EnumField('StateValueValuesEnum', 2)


class SerdeInfo(_messages.Message):
  r"""Serialization and deserialization information.

  Enums:
    SerdeTypeValueValuesEnum: Optional. The serde type.

  Messages:
    ParametersValue: Optional. Parameters of the serde. The maximum size is
      10Kib.

  Fields:
    description: Optional. Description of the serde. The maximum length is
      4000 characters.
    deserializerClass: Optional. The fully qualified Java class name of the
      deserializer. The maximum length is 4000 characters.
    name: Required. Name of the SerDe. Table name by default. The maximum
      length is 128 characters.
    parameters: Optional. Parameters of the serde. The maximum size is 10Kib.
    serdeType: Optional. The serde type.
    serializationLib: Required. The fully qualified Java class name of the
      serialization library. The maximum length is 4000 characters.
    serializerClass: Optional. The fully qualified Java class name of the
      serializer. The maximum length is 4000 characters.
  """

  class SerdeTypeValueValuesEnum(_messages.Enum):
    r"""Optional. The serde type.

    Values:
      SERDE_TYPE_UNSPECIFIED: The serde type is not specified.
      HIVE: Hive.
      SCHEMA_REGISTRY: Schema registry.
    """
    SERDE_TYPE_UNSPECIFIED = 0
    HIVE = 1
    SCHEMA_REGISTRY = 2

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ParametersValue(_messages.Message):
    r"""Optional. Parameters of the serde. The maximum size is 10Kib.

    Messages:
      AdditionalProperty: An additional property for a ParametersValue object.

    Fields:
      additionalProperties: Additional properties of type ParametersValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ParametersValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  description = _messages.StringField(1)
  deserializerClass = _messages.StringField(2)
  name = _messages.StringField(3)
  parameters = _messages.MessageField('ParametersValue', 4)
  serdeType = _messages.EnumField('SerdeTypeValueValuesEnum', 5)
  serializationLib = _messages.StringField(6)
  serializerClass = _messages.StringField(7)


class SkewedColumnValue(_messages.Message):
  r"""The skewed column values.

  Fields:
    values: Required. The skewed column values. The maximum length is 256
      characters per value.
  """

  values = _messages.StringField(1, repeated=True)


class SkewedInfo(_messages.Message):
  r"""Stores all the information about skewed table.

  Fields:
    skewedColNames: Required. The column names that are skewed. The maximum
      length is 256 characters per column name.
    skewedColValues: Required. The skewed column values.
    skewedKeyValuesLocations: Required. The skewed key values locations.
  """

  skewedColNames = _messages.StringField(1, repeated=True)
  skewedColValues = _messages.MessageField('SkewedColumnValue', 2, repeated=True)
  skewedKeyValuesLocations = _messages.MessageField('SkewedKeyValuesLocation', 3, repeated=True)


class SkewedKeyValuesLocation(_messages.Message):
  r"""The skewed key values and their corresponding location.

  Fields:
    location: Required. The location of the skewed column values. The maximum
      length is 4000 characters.
    values: Required. The skewed column values. The maximum length is 256
      characters per value.
  """

  location = _messages.StringField(1)
  values = _messages.StringField(2, repeated=True)


class StandardQueryParameters(_messages.Message):
  r"""Query parameters accepted by all methods.

  Enums:
    FXgafvValueValuesEnum: V1 error format.
    AltValueValuesEnum: Data format for response.

  Fields:
    f__xgafv: V1 error format.
    access_token: OAuth access token.
    alt: Data format for response.
    callback: JSONP
    fields: Selector specifying which fields to include in a partial response.
    key: API key. Your API key identifies your project and provides you with
      API access, quota, and reports. Required unless you provide an OAuth 2.0
      token.
    oauth_token: OAuth 2.0 token for the current user.
    prettyPrint: Returns response with indentations and line breaks.
    quotaUser: Available to use for quota purposes for server-side
      applications. Can be any arbitrary string assigned to a user, but should
      not exceed 40 characters.
    trace: A tracing token of the form "token:<tokenid>" to include in api
      requests.
    uploadType: Legacy upload protocol for media (e.g. "media", "multipart").
    upload_protocol: Upload protocol for media (e.g. "raw", "multipart").
  """

  class AltValueValuesEnum(_messages.Enum):
    r"""Data format for response.

    Values:
      json: Responses with Content-Type of application/json
      media: Media download with context-dependent Content-Type
      proto: Responses with Content-Type of application/x-protobuf
    """
    json = 0
    media = 1
    proto = 2

  class FXgafvValueValuesEnum(_messages.Enum):
    r"""V1 error format.

    Values:
      _1: v1 error format
      _2: v2 error format
    """
    _1 = 0
    _2 = 1

  f__xgafv = _messages.EnumField('FXgafvValueValuesEnum', 1)
  access_token = _messages.StringField(2)
  alt = _messages.EnumField('AltValueValuesEnum', 3, default='json')
  callback = _messages.StringField(4)
  fields = _messages.StringField(5)
  key = _messages.StringField(6)
  oauth_token = _messages.StringField(7)
  prettyPrint = _messages.BooleanField(8, default=True)
  quotaUser = _messages.StringField(9)
  trace = _messages.StringField(10)
  uploadType = _messages.StringField(11)
  upload_protocol = _messages.StringField(12)


class StorageDescriptor(_messages.Message):
  r"""Contains information about the physical storage of the table data.

  Messages:
    ParametersValue: Optional. Key-value pairs for the storage descriptor. The
      maximum size is 10Kib.

  Fields:
    bucketCols: Optional. Reducer grouping columns and clustering columns and
      bucketing columns
    columns: Required. Specifies the columns of the table.
    compressed: Optional. Whether the table is compressed.
    inputFormat: Optional. The fully qualified Java class name of the input
      format. The maximum length is 4000 characters.
    locationUri: Optional. The Cloud storage uri where the table is located.
      Defaults to `/`. The maximum length is 4000 characters.
    numBuckets: Optional. The number of buckets in the table.
    outputFormat: Optional. The fully qualified Java class name of the output
      format. The maximum length is 4000 characters.
    parameters: Optional. Key-value pairs for the storage descriptor. The
      maximum size is 10Kib.
    serdeInfo: Optional. Serialization and deserialization information.
    skewedInfo: Optional. Table data skew information.
    sortCols: Optional. Sort order of the data in each bucket
    storedAsSubDirs: Optional. Whether the table is stored as sub directories.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ParametersValue(_messages.Message):
    r"""Optional. Key-value pairs for the storage descriptor. The maximum size
    is 10Kib.

    Messages:
      AdditionalProperty: An additional property for a ParametersValue object.

    Fields:
      additionalProperties: Additional properties of type ParametersValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ParametersValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  bucketCols = _messages.StringField(1, repeated=True)
  columns = _messages.MessageField('FieldSchema', 2, repeated=True)
  compressed = _messages.BooleanField(3)
  inputFormat = _messages.StringField(4)
  locationUri = _messages.StringField(5)
  numBuckets = _messages.IntegerField(6, variant=_messages.Variant.INT32)
  outputFormat = _messages.StringField(7)
  parameters = _messages.MessageField('ParametersValue', 8)
  serdeInfo = _messages.MessageField('SerdeInfo', 9)
  skewedInfo = _messages.MessageField('SkewedInfo', 10)
  sortCols = _messages.MessageField('Order', 11, repeated=True)
  storedAsSubDirs = _messages.BooleanField(12)


class UpdatePartitionRequest(_messages.Message):
  r"""Request message for UpdatePartition.

  Fields:
    partition: Required. The partition to be updated.
    updateMask: Optional. The list of fields to update.
  """

  partition = _messages.MessageField('Partition', 1)
  updateMask = _messages.StringField(2)


encoding.AddCustomJsonFieldMapping(
    StandardQueryParameters, 'f__xgafv', '$.xgafv')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_1', '1')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_2', '2')



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Library for creating pull request related messages."""


import json


def CreateCommitMessage(finding_data, member_name):
  """Creates a commit message.

  Args:
    finding_data: Finding data in JSON format.
    member_name: The name of the member to be added to the commit message.

  Returns:
    A string containing the commit message.
  """
  finding_data = json.loads(finding_data)
  finding_result = finding_data["listFindingsResults"][0]
  finding_category = finding_result["finding"][
      "category"
  ]
  crm_node = finding_result["resource"]["displayName"]
  finding_name = finding_result["finding"]["name"]

  return (
      f"Fixing {finding_name} of category {finding_category} for"
      f" {member_name} in {crm_node}"
  )


def CreatePRMessage(finding_data):
  """Creates a commit message for a pull request.

  Args:
    finding_data: Finding data in JSON format.

  Returns:
    A string containing the Pull Request(PR) message.
  """
  finding_data = json.loads(finding_data)
  finding_result = finding_data["listFindingsResults"][0]
  finding_category = finding_result["finding"][
      "category"
  ]
  crm_node = finding_result["resource"]["displayName"]
  finding_name = finding_result["finding"]["name"]

  return (
      f"Fixing Finding:{finding_name} of Category:{finding_category}"
      f" in {crm_node}"
  )



# -*- coding: utf-8 -*- # Lint as: python3
# Copyright 2020 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The environments command group for the Apigee CLI."""


from googlecloudsdk.calliope import base


class Environments(base.Group):
  """Manage Apigee environments."""

  detailed_help = {
      "EXAMPLES":
          """\
  To list all environments for the active Cloud Platform project, run:

      $ {command} list

  To get a JSON array of all environments in an Apigee organization called
  ``my-org'', run:

      $ {command} list --organization=my-org --format=json
  """}



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Common flags for network-actions resources."""


import textwrap

from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope.concepts import concepts
from googlecloudsdk.command_lib.network_actions import util
from googlecloudsdk.command_lib.util.apis import yaml_data
from googlecloudsdk.command_lib.util.concepts import concept_parsers
from googlecloudsdk.command_lib.util.concepts import presentation_specs


def AddWasmPluginResource(parser, api_version, message):
  wasm_plugin_data = yaml_data.ResourceYAMLData.FromPath(
      'network_actions.wasmPlugin'
  )
  concept_parsers.ConceptParser(
      [
          presentation_specs.ResourcePresentationSpec(
              'wasm_plugin',
              concepts.ResourceSpec.FromYaml(
                  wasm_plugin_data.GetData(),
                  api_version=api_version,
              ),
              message,
              required=True,
          )
      ],
  ).AddToParser(parser)


def AddDescriptionFlag(parser):
  parser.add_argument(
      '--description',
      help='A human-readable description of the resource.')


def AddLogConfigFlag(parser):
  parser.add_argument(
      '--log-config',
      action='append',
      type=util.LogConfig(),
      required=False,
      metavar='LOG_CONFIG',
      help=textwrap.dedent("""\
        Logging options for the activity performed by this plugin.
        The following options can be set:
        * `enable`: whether to enable logging. If `log-config` flag is set,
          `enable` option is required.

        * `sample-rate`: configures the sampling rate of activity logs, where
          `1.0` means all logged activity is reported and `0.0` means no
          activity is reported. The default value is `1.0`, and the value of
          the field must be in range `0` to `1` (inclusive).

        * `min-log-level`: specifies the lowest level of the logs that
          should be exported to Cloud Logging. The default value is `INFO`.

        Example usage:
        `--log-config=enable=True,sample-rate=0.5,min-log-level=INFO
        --log_config=enable=False`
        """),
  )


def AddVersionFlag(parser, version_message):
  parser.add_argument('--main-version', help=version_message)


def AddImageFlag(parser):
  parser.add_argument(
      '--image',
      help=textwrap.dedent("""\
          URI of the container image containing the plugin's Wasm module,
          stored in the Artifact Registry."""),
  )


def AddPluginConfigFlag(parser):
  """Adds flags defining plugin config."""
  plugin_config_group = parser.add_group(
      mutex=True,
      required=False,
      help="""Configuration for the plugin, provided at runtime by the
              `on_configure` function (Rust Proxy-Wasm SDK) or the
              `onConfigure` method (C++ Proxy-Wasm SDK).""",
  )
  plugin_config_group.add_argument(
      '--plugin-config',
      required=False,
      help="""Plugin runtime configuration in the textual format."""
  )
  plugin_config_group.add_argument(
      '--plugin-config-file',
      required=False,
      type=arg_parsers.FileContents(binary=True),
      help="""Path to a local file containing the plugin runtime
              configuration."""
  )
  plugin_config_group.add_argument(
      '--plugin-config-uri',
      required=False,
      help="""URI of the container image containing the plugin's runtime
              configuration, stored in the Artifact Registry."""
  )


def AddWasmPluginVersionArgs(parser, version_message):
  wasm_plugin_version_group = parser.add_group(mutex=False, required=False)
  AddVersionFlag(wasm_plugin_version_group, version_message)
  AddImageFlag(wasm_plugin_version_group)
  AddPluginConfigFlag(wasm_plugin_version_group)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

import proto  # type: ignore

from cloudsdk.google.protobuf import duration_pb2  # type: ignore
from googlecloudsdk.generated_clients.gapic_clients.run_v2.types import k8s_min
from googlecloudsdk.generated_clients.gapic_clients.run_v2.types import vendor_settings


__protobuf__ = proto.module(
    package='google.cloud.run.v2',
    manifest={
        'RevisionTemplate',
    },
)


class RevisionTemplate(proto.Message):
    r"""RevisionTemplate describes the data a revision should have
    when created from a template.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        revision (str):
            Optional. The unique name for the revision.
            If this field is omitted, it will be
            automatically generated based on the Service
            name.
        labels (MutableMapping[str, str]):
            Optional. Unstructured key value map that can be used to
            organize and categorize objects. User-provided labels are
            shared with Google's billing system, so they can be used to
            filter, or break down billing charges by team, component,
            environment, state, etc. For more information, visit
            https://cloud.google.com/resource-manager/docs/creating-managing-labels
            or https://cloud.google.com/run/docs/configuring/labels.

            .. raw:: html

                <p>Cloud Run API v2 does not support labels with `run.googleapis.com`,
                `cloud.googleapis.com`, `serving.knative.dev`, or `autoscaling.knative.dev`
                namespaces, and they will be rejected. All system labels in v1 now have a
                corresponding field in v2 RevisionTemplate.
        annotations (MutableMapping[str, str]):
            Optional. Unstructured key value map that may be set by
            external tools to store and arbitrary metadata. They are not
            queryable and should be preserved when modifying objects.

            .. raw:: html

                <p>Cloud Run API v2 does not support annotations with `run.googleapis.com`,
                `cloud.googleapis.com`, `serving.knative.dev`, or `autoscaling.knative.dev`
                namespaces, and they will be rejected. All system annotations in v1 now
                have a corresponding field in v2 RevisionTemplate.

            .. raw:: html

                <p>This field follows Kubernetes annotations' namespacing, limits, and
                rules.
        scaling (googlecloudsdk.generated_clients.gapic_clients.run_v2.types.RevisionScaling):
            Optional. Scaling settings for this Revision.
        vpc_access (googlecloudsdk.generated_clients.gapic_clients.run_v2.types.VpcAccess):
            Optional. VPC Access configuration to use for
            this Revision. For more information, visit
            https://cloud.google.com/run/docs/configuring/connecting-vpc.
        timeout (google.protobuf.duration_pb2.Duration):
            Optional. Max allowed time for an instance to
            respond to a request.
        service_account (str):
            Optional. Email address of the IAM service
            account associated with the revision of the
            service. The service account represents the
            identity of the running revision, and determines
            what permissions the revision has. If not
            provided, the revision will use the project's
            default service account.
        containers (MutableSequence[googlecloudsdk.generated_clients.gapic_clients.run_v2.types.Container]):
            Holds the list which define the units of
            execution for this Revision.
        volumes (MutableSequence[googlecloudsdk.generated_clients.gapic_clients.run_v2.types.Volume]):
            Optional. A list of Volumes to make available
            to containers.
        execution_environment (googlecloudsdk.generated_clients.gapic_clients.run_v2.types.ExecutionEnvironment):
            Optional. The sandbox environment to host
            this Revision.
        encryption_key (str):
            A reference to a customer managed encryption
            key (CMEK) to use to encrypt this container
            image. For more information, go to
            https://cloud.google.com/run/docs/securing/using-cmek
        max_instance_request_concurrency (int):
            Optional. Sets the maximum number of requests that each
            serving instance can receive. If not specified or 0,
            concurrency defaults to 80 when requested ``CPU >= 1`` and
            defaults to 1 when requested ``CPU < 1``.
        service_mesh (googlecloudsdk.generated_clients.gapic_clients.run_v2.types.ServiceMesh):
            Optional. Enables service mesh connectivity.
        encryption_key_revocation_action (googlecloudsdk.generated_clients.gapic_clients.run_v2.types.EncryptionKeyRevocationAction):
            Optional. The action to take if the
            encryption key is revoked.
        encryption_key_shutdown_duration (google.protobuf.duration_pb2.Duration):
            Optional. If encryption_key_revocation_action is SHUTDOWN,
            the duration before shutting down all instances. The minimum
            increment is 1 hour.
        session_affinity (bool):
            Optional. Enable session affinity.
        health_check_disabled (bool):
            Optional. Disables health checking containers
            during deployment.
        node_selector (googlecloudsdk.generated_clients.gapic_clients.run_v2.types.NodeSelector):
            Optional. The node selector for the revision
            template.
        gpu_zonal_redundancy_disabled (bool):
            Optional. True if GPU zonal redundancy is
            disabled on this revision.

            This field is a member of `oneof`_ ``_gpu_zonal_redundancy_disabled``.
        client (str):
            Optional. Arbitrary identifier for the API
            client.
        client_version (str):
            Optional. Arbitrary version identifier for
            the API client.
    """

    revision: str = proto.Field(
        proto.STRING,
        number=1,
    )
    labels: MutableMapping[str, str] = proto.MapField(
        proto.STRING,
        proto.STRING,
        number=2,
    )
    annotations: MutableMapping[str, str] = proto.MapField(
        proto.STRING,
        proto.STRING,
        number=3,
    )
    scaling: vendor_settings.RevisionScaling = proto.Field(
        proto.MESSAGE,
        number=4,
        message=vendor_settings.RevisionScaling,
    )
    vpc_access: vendor_settings.VpcAccess = proto.Field(
        proto.MESSAGE,
        number=6,
        message=vendor_settings.VpcAccess,
    )
    timeout: duration_pb2.Duration = proto.Field(
        proto.MESSAGE,
        number=8,
        message=duration_pb2.Duration,
    )
    service_account: str = proto.Field(
        proto.STRING,
        number=9,
    )
    containers: MutableSequence[k8s_min.Container] = proto.RepeatedField(
        proto.MESSAGE,
        number=10,
        message=k8s_min.Container,
    )
    volumes: MutableSequence[k8s_min.Volume] = proto.RepeatedField(
        proto.MESSAGE,
        number=11,
        message=k8s_min.Volume,
    )
    execution_environment: vendor_settings.ExecutionEnvironment = proto.Field(
        proto.ENUM,
        number=13,
        enum=vendor_settings.ExecutionEnvironment,
    )
    encryption_key: str = proto.Field(
        proto.STRING,
        number=14,
    )
    max_instance_request_concurrency: int = proto.Field(
        proto.INT32,
        number=15,
    )
    service_mesh: vendor_settings.ServiceMesh = proto.Field(
        proto.MESSAGE,
        number=16,
        message=vendor_settings.ServiceMesh,
    )
    encryption_key_revocation_action: vendor_settings.EncryptionKeyRevocationAction = proto.Field(
        proto.ENUM,
        number=17,
        enum=vendor_settings.EncryptionKeyRevocationAction,
    )
    encryption_key_shutdown_duration: duration_pb2.Duration = proto.Field(
        proto.MESSAGE,
        number=18,
        message=duration_pb2.Duration,
    )
    session_affinity: bool = proto.Field(
        proto.BOOL,
        number=19,
    )
    health_check_disabled: bool = proto.Field(
        proto.BOOL,
        number=20,
    )
    node_selector: vendor_settings.NodeSelector = proto.Field(
        proto.MESSAGE,
        number=21,
        message=vendor_settings.NodeSelector,
    )
    gpu_zonal_redundancy_disabled: bool = proto.Field(
        proto.BOOL,
        number=24,
        optional=True,
    )
    client: str = proto.Field(
        proto.STRING,
        number=28,
    )
    client_version: str = proto.Field(
        proto.STRING,
        number=29,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Generic functions for listing commands."""


import abc
import enum

from googlecloudsdk.api_lib.storage import cloud_api
from googlecloudsdk.command_lib.storage import errors
from googlecloudsdk.command_lib.storage import folder_util
from googlecloudsdk.command_lib.storage import plurality_checkable_iterator
from googlecloudsdk.command_lib.storage import wildcard_iterator
from googlecloudsdk.command_lib.storage.resources import resource_reference
from googlecloudsdk.command_lib.storage.resources import shim_format_util
import six


def check_and_convert_to_readable_sizes(
    size, readable_sizes=False, use_gsutil_style=False
):
  if readable_sizes and size is not None:
    return shim_format_util.get_human_readable_byte_value(
        size, use_gsutil_style=use_gsutil_style
    )
  else:
    # Also handles None values.
    return six.text_type(size)


class DisplayDetail(enum.Enum):
  """Level of detail to display about items being printed."""

  SHORT = 1
  LONG = 2
  FULL = 3
  JSON = 4


def _translate_display_detail_to_fields_scope(
    display_detail, is_bucket_listing
):
  """Translates display details to fields scope equivalent.

  Args:
    display_detail (DisplayDetail): Argument to translate.
    is_bucket_listing (bool): Buckets require special handling.

  Returns:
    cloud_api.FieldsScope appropriate for the resources and display detail.
  """
  # Long listing is the same as normal listing for buckets.
  if display_detail == DisplayDetail.LONG and is_bucket_listing:
    return cloud_api.FieldsScope.SHORT

  display_detail_to_fields_scope = {
      DisplayDetail.SHORT: cloud_api.FieldsScope.SHORT,
      DisplayDetail.LONG: cloud_api.FieldsScope.NO_ACL,
      DisplayDetail.FULL: cloud_api.FieldsScope.FULL,
      DisplayDetail.JSON: cloud_api.FieldsScope.FULL,
  }
  return display_detail_to_fields_scope[display_detail]


class BaseFormatWrapper(six.with_metaclass(abc.ABCMeta)):
  """For formatting how items are printed when listed.

  Child classes should set _header_wrapper and _object_wrapper.

  Attributes:
    resource (resource_reference.Resource): Item to be formatted for printing.
  """

  def __init__(
      self,
      resource,
      display_detail=DisplayDetail.SHORT,
      full_formatter=None,
      include_etag=None,
      object_state=None,
      readable_sizes=False,
      use_gsutil_style=False,
  ):
    """Initializes wrapper instance.

    Args:
      resource (resource_reference.Resource): Item to be formatted for printing.
      display_detail (DisplayDetail): Level of metadata detail for printing.
      full_formatter (base.FullResourceFormatter): Printing formatter used witch
        FULL DisplayDetail.
      include_etag (bool): Display etag string of resource.
      object_state (cloud_api.ObjectState): What versions of an object to query.
      readable_sizes (bool): Convert bytes to a more human readable format for
        long lising. For example, print 1024B as 1KiB.
      use_gsutil_style (bool): Outputs closer to the style of the gsutil CLI.
    """
    self.resource = resource
    self._display_detail = display_detail
    self._full_formatter = full_formatter
    self._include_etag = include_etag
    self._object_state = object_state
    self._readable_sizes = readable_sizes
    self._use_gsutil_style = use_gsutil_style

  def _check_and_handles_versions(self):
    show_version_in_url = self._object_state in (
        cloud_api.ObjectState.LIVE_AND_NONCURRENT,
        cloud_api.ObjectState.SOFT_DELETED,
    )
    if show_version_in_url:
      url_string = self.resource.storage_url.url_string
      metageneration_string = '  metageneration={}'.format(
          six.text_type(self.resource.metageneration)
      )
    else:
      url_string = self.resource.storage_url.versionless_url_string
      metageneration_string = ''
    return (url_string, metageneration_string)


class NullFormatWrapper(BaseFormatWrapper):
  """For formatting how containers are printed as headers when listed."""

  def __init__(
      self,
      resource,
      container_size=None,
      display_detail=None,
      full_formatter=None,
      include_etag=None,
      object_state=None,
      readable_sizes=None,
      use_gsutil_style=None,
  ):
    super(NullFormatWrapper, self).__init__(resource)
    del (
        container_size,
        display_detail,
        full_formatter,
        include_etag,
        object_state,
        readable_sizes,
        use_gsutil_style,
    )  # Unused.

  def __str__(self):
    return ''


class BaseListExecutor(six.with_metaclass(abc.ABCMeta)):
  """Abstract base class for list executors (e.g. for ls and du)."""

  def __init__(
      self,
      cloud_urls,
      buckets_flag=False,
      display_detail=DisplayDetail.SHORT,
      exclude_patterns=None,
      fetch_encrypted_object_hashes=False,
      halt_on_empty_response=True,
      include_etag=False,
      include_managed_folders=False,
      next_page_token=None,
      object_state=None,
      readable_sizes=False,
      recursion_flag=False,
      total=False,
      use_gsutil_style=False,
      zero_terminator=False,
      soft_deleted_buckets=False,
      list_filter=None,
  ):
    """Initializes executor.

    Args:
      cloud_urls ([storage_url.CloudUrl]): List of non-local filesystem URLs.
      buckets_flag (bool): If given a bucket URL, only return matching buckets
        ignoring normal recursion rules.
      display_detail (DisplayDetail): Determines level of metadata printed.
      exclude_patterns (Patterns|None): Don't return resources whose URLs or
        local file paths matched these regex patterns.
      fetch_encrypted_object_hashes (bool): Fall back to GET requests for
        encrypted objects in order to fetch their hash values.
      halt_on_empty_response (bool): Stops querying after empty list response.
        See CloudApi for details.
      include_etag (bool): Print etag string of resource, depending on other
        settings.
      include_managed_folders (bool): Includes managed folders in list results.
      next_page_token (str|None): Used to resume LIST calls.
      object_state (cloud_api.ObjectState): Versions of objects to query.
      readable_sizes (bool): Convert bytes to a more human readable format for
        long lising. For example, print 1024B as 1KiB.
      recursion_flag (bool): Recurse through all containers and format all
        container headers.
      total (bool): Add up the total size of all input sources.
      use_gsutil_style (bool): Outputs closer to the style of the gsutil CLI.
      zero_terminator (bool): Use null byte instead of newline as line
        terminator.
      soft_deleted_buckets (bool): If true, soft deleted buckets will be listed.
      list_filter (str|None): Performs list call with specified filter string.
    """
    self._cloud_urls = cloud_urls
    self._buckets_flag = buckets_flag
    self._display_detail = display_detail
    self._exclude_patterns = exclude_patterns
    self._fetch_encrypted_object_hashes = fetch_encrypted_object_hashes
    self._halt_on_empty_response = halt_on_empty_response
    self._include_etag = include_etag
    self._include_managed_folders = include_managed_folders
    self._next_page_token = next_page_token
    self._object_state = object_state
    self._readable_sizes = readable_sizes
    self._recursion_flag = recursion_flag
    self._total = total
    self._use_gsutil_style = use_gsutil_style
    self._zero_terminator = zero_terminator
    self._soft_deleted_buckets = soft_deleted_buckets
    self._list_filter = list_filter

    self._full_formatter = None
    # Null wrappers print nothing
    self._header_wrapper = NullFormatWrapper
    self._container_summary_wrapper = NullFormatWrapper
    self._object_wrapper = NullFormatWrapper

  def _get_container_iterator(self, container_cloud_url, recursion_level):
    """For recursing into and retrieving the contents of a container.

    Args:
      container_cloud_url (storage_url.CloudUrl): Container URL for recursing
        into.
      recursion_level (int): Determines if iterator should keep recursing.

    Returns:
      BaseFormatWrapper generator.
    """
    # End URL with '/*', so WildcardIterator won't filter out its contents.
    new_cloud_url = container_cloud_url.join('*')
    fields_scope = _translate_display_detail_to_fields_scope(
        self._display_detail, is_bucket_listing=False
    )
    if self._include_managed_folders:
      # Not using LIST_WITH_OBJECTS here since it adds extra managed folder
      # GET/LIST calls. This is helpful if a delimiter isn't provided with the
      # request, but since this call is used for recursion, which traverses each
      # layer of hierarchy using a delimiter, we do not need these calls.
      managed_folder_setting = folder_util.ManagedFolderSetting.LIST_AS_PREFIXES
    else:
      managed_folder_setting = folder_util.ManagedFolderSetting.DO_NOT_LIST

    iterator = wildcard_iterator.CloudWildcardIterator(
        new_cloud_url,
        error_on_missing_key=False,
        exclude_patterns=self._exclude_patterns,
        fetch_encrypted_object_hashes=self._fetch_encrypted_object_hashes,
        fields_scope=fields_scope,
        halt_on_empty_response=self._halt_on_empty_response,
        managed_folder_setting=managed_folder_setting,
        folder_setting=folder_util.FolderSetting.LIST_AS_PREFIXES,
        next_page_token=self._next_page_token,
        object_state=self._object_state,
        soft_deleted_buckets=self._soft_deleted_buckets,
        list_filter=self._list_filter,
    )
    return self._recursion_helper(iterator, recursion_level)

  def _recursion_helper(
      self, iterator, recursion_level, print_top_level_container=True
  ):
    """For retrieving resources from URLs that potentially contain wildcards.

    Args:
      iterator (Iterable[resource_reference.Resource]): For recursing through.
      recursion_level (int): Integer controlling how deep the listing recursion
        goes. "1" is the default, mimicking the actual OS ls, which lists the
        contents of the first level of matching subdirectories. Call with
        "float('inf')" for listing everything available.
      print_top_level_container (bool): Used by `du` to skip printing the top
        level bucket

    Yields:
      BaseFormatWrapper generator.
    """
    for resource in iterator:
      # Check if we need to display contents of a container.
      if (
          resource_reference.is_container_or_has_container_url(resource)
          and recursion_level > 0
      ):
        if self._header_wrapper != NullFormatWrapper:
          yield self._header_wrapper(
              resource,
              display_detail=self._display_detail,
              include_etag=self._include_etag,
              object_state=self._object_state,
              readable_sizes=self._readable_sizes,
              full_formatter=self._full_formatter,
              use_gsutil_style=self._use_gsutil_style,
          )

        container_size = 0
        # Get container contents by adding wildcard to URL.
        nested_iterator = self._get_container_iterator(
            resource.storage_url, recursion_level - 1
        )
        for nested_resource in nested_iterator:
          if (
              self._container_summary_wrapper != NullFormatWrapper
              and print_top_level_container
          ):
            container_size += getattr(nested_resource.resource, 'size', 0)
          yield nested_resource
        # Print directories after objects for `du` command
        if (
            self._container_summary_wrapper != NullFormatWrapper
            and print_top_level_container
        ):
          yield self._container_summary_wrapper(
              resource=resource,
              container_size=container_size,
              object_state=self._object_state,
              readable_sizes=self._readable_sizes,
          )

      else:
        # Resource wasn't a container we can recurse into, so just yield it.
        yield self._object_wrapper(
            resource,
            display_detail=self._display_detail,
            full_formatter=self._full_formatter,
            include_etag=self._include_etag,
            object_state=self._object_state,
            readable_sizes=self._readable_sizes,
            use_gsutil_style=self._use_gsutil_style,
        )

  def _print_summary_for_top_level_url(
      self, resource_url, only_display_buckets, object_count, total_bytes
  ):
    del self, resource_url, only_display_buckets, object_count, total_bytes

  def _print_row_list(
      self, resource_wrappers, resource_url, only_display_buckets
  ):
    """Prints ResourceWrapper objects in list with custom row formatting."""
    object_count = total_bytes = 0
    terminator = '\0' if self._zero_terminator else '\n'
    for i, resource_wrapper in enumerate(resource_wrappers):
      resource_wrapper_string = six.text_type(resource_wrapper)
      if isinstance(
          resource_wrapper.resource, resource_reference.ObjectResource
      ):
        # For printing long listing data summary.
        object_count += 1
        total_bytes += resource_wrapper.resource.size or 0
      if not resource_wrapper_string:
        continue
      if i == 0 and resource_wrapper and resource_wrapper_string[0] == '\n':
        # First print should not begin with a line break, which can happen
        # for headers.
        print(resource_wrapper_string[1:], end=terminator)
      else:
        print(resource_wrapper_string, end=terminator)

    self._print_summary_for_top_level_url(
        resource_url=resource_url,
        only_display_buckets=only_display_buckets,
        object_count=object_count,
        total_bytes=total_bytes,
    )

    return total_bytes

  def _should_only_display_buckets(self, raw_cloud_url):
    # Ls received a provider URL ("gs://") -> List all buckets.
    # Received buckets flag and bucket URL -> List matching buckets, ignoring
    # recursion.
    return raw_cloud_url.is_provider() or (
        self._buckets_flag and raw_cloud_url.is_bucket()
    )

  def _list_url(self, raw_cloud_url):
    """Recursively create wildcard iterators to print all relevant items."""
    fields_scope = _translate_display_detail_to_fields_scope(
        self._display_detail, is_bucket_listing=raw_cloud_url.is_provider()
    )

    if self._include_managed_folders:
      managed_folder_setting = folder_util.ManagedFolderSetting.LIST_AS_PREFIXES
    else:
      managed_folder_setting = folder_util.ManagedFolderSetting.DO_NOT_LIST

    resources = plurality_checkable_iterator.PluralityCheckableIterator(
        wildcard_iterator.CloudWildcardIterator(
            raw_cloud_url,
            error_on_missing_key=False,
            exclude_patterns=self._exclude_patterns,
            fetch_encrypted_object_hashes=self._fetch_encrypted_object_hashes,
            fields_scope=fields_scope,
            get_bucket_metadata=self._buckets_flag,
            halt_on_empty_response=self._halt_on_empty_response,
            managed_folder_setting=managed_folder_setting,
            folder_setting=folder_util.FolderSetting.LIST_AS_PREFIXES,
            next_page_token=self._next_page_token,
            object_state=self._object_state,
            soft_deleted_buckets=self._soft_deleted_buckets,
            list_filter=self._list_filter,
        )
    )

    if resources.is_empty():
      raise errors.InvalidUrlError('One or more URLs matched no objects.')

    only_display_buckets = self._should_only_display_buckets(raw_cloud_url)
    if only_display_buckets:
      resources_wrappers = self._recursion_helper(resources, recursion_level=0)
    elif self._recursion_flag and '**' not in raw_cloud_url.url_string:
      # "**" overrides recursive flag.
      print_top_level_container = True
      if raw_cloud_url.is_bucket():
        print_top_level_container = False
      resources_wrappers = self._recursion_helper(
          resources, float('inf'), print_top_level_container
      )
    elif not resources.is_plural() and (
        resource_reference.is_container_or_has_container_url(resources.peek())
    ):
      # One container was returned by the query, in which case we show
      # its contents.
      resources_wrappers = self._get_container_iterator(
          resources.peek().storage_url, recursion_level=0
      )
    else:
      resources_wrappers = self._recursion_helper(resources, recursion_level=1)

    size_in_bytes = 0
    if self._display_detail == DisplayDetail.JSON:
      self._print_json_list(resources_wrappers)
    else:
      size_in_bytes = self._print_row_list(
          resources_wrappers, raw_cloud_url, only_display_buckets
      )
    return size_in_bytes

  def _print_total(self, all_sources_total_bytes):
    del all_sources_total_bytes

  # Method is intended to be implemented by subclass to print bucket headers for
  # `ls` command.
  def _print_bucket_header(self, url):
    del url

  def list_urls(self):
    all_sources_total_bytes = 0
    for url in self._cloud_urls:
      if url.is_bucket():
        self._print_bucket_header(url)
      if self._total:
        all_sources_total_bytes += self._list_url(url)
      else:
        self._list_url(url)

    if self._total:
      self._print_total(all_sources_total_bytes)



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# NOTE: This file is autogenerated and should not be edited by hand.
# AUTOGEN_CLI_VERSION: HEAD
"""Manage Discovered Api Operation resources."""

from googlecloudsdk.calliope import base
from surface.apihub.discovered_api_observations.discovered_api_operations import _init_extensions as extensions


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
@base.Autogenerated
class DiscoveredApiOperationsAlpha(extensions.DiscoveredApiOperationsAlpha):
  """Manage Discovered Api Operation resources."""


@base.ReleaseTracks(base.ReleaseTrack.GA)
@base.Autogenerated
class DiscoveredApiOperationsGa(extensions.DiscoveredApiOperationsGa):
  """Manage Discovered Api Operation resources."""



"""Generated message classes for cloudshell version v1alpha1.

Allows users to start, configure, and connect to interactive shell sessions
running in the cloud.
"""
# NOTE: This file is autogenerated and should not be edited by hand.


from apitools.base.protorpclite import messages as _messages
from apitools.base.py import encoding
from apitools.base.py import extra_types


package = 'cloudshell'


class AuthorizeEnvironmentRequest(_messages.Message):
  r"""Request message for AuthorizeEnvironment.

  Fields:
    accessToken: The OAuth access token that should be sent to the
      environment.
    expireTime: The time when the credentials expire. If not set, defaults to
      one hour from when the server received the request.
    idToken: The OAuth ID token that should be sent to the environment.
  """

  accessToken = _messages.StringField(1)
  expireTime = _messages.StringField(2)
  idToken = _messages.StringField(3)


class CloudshellUsersEnvironmentsAuthorizeRequest(_messages.Message):
  r"""A CloudshellUsersEnvironmentsAuthorizeRequest object.

  Fields:
    authorizeEnvironmentRequest: A AuthorizeEnvironmentRequest resource to be
      passed as the request body.
    name: Name of the resource that should receive the credentials, for
      example `users/me/environments/default` or
      `users/someone@example.com/environments/default`.
  """

  authorizeEnvironmentRequest = _messages.MessageField('AuthorizeEnvironmentRequest', 1)
  name = _messages.StringField(2, required=True)


class CloudshellUsersEnvironmentsGetRequest(_messages.Message):
  r"""A CloudshellUsersEnvironmentsGetRequest object.

  Fields:
    name: Name of the requested resource, for example
      `users/me/environments/default` or
      `users/someone@example.com/environments/default`.
  """

  name = _messages.StringField(1, required=True)


class CloudshellUsersEnvironmentsPatchRequest(_messages.Message):
  r"""A CloudshellUsersEnvironmentsPatchRequest object.

  Fields:
    environment: A Environment resource to be passed as the request body.
    name: Name of the resource to be updated, for example
      `users/me/environments/default` or
      `users/someone@example.com/environments/default`.
    updateMask: Mask specifying which fields in the environment should be
      updated.
  """

  environment = _messages.MessageField('Environment', 1)
  name = _messages.StringField(2, required=True)
  updateMask = _messages.StringField(3)


class CloudshellUsersEnvironmentsPublicKeysCreateRequest(_messages.Message):
  r"""A CloudshellUsersEnvironmentsPublicKeysCreateRequest object.

  Fields:
    createPublicKeyRequest: A CreatePublicKeyRequest resource to be passed as
      the request body.
    parent: Parent resource name, e.g. `users/me/environments/default`.
  """

  createPublicKeyRequest = _messages.MessageField('CreatePublicKeyRequest', 1)
  parent = _messages.StringField(2, required=True)


class CloudshellUsersEnvironmentsPublicKeysDeleteRequest(_messages.Message):
  r"""A CloudshellUsersEnvironmentsPublicKeysDeleteRequest object.

  Fields:
    name: Name of the resource to be deleted, e.g.
      `users/me/environments/default/publicKeys/my-key`.
  """

  name = _messages.StringField(1, required=True)


class CloudshellUsersEnvironmentsStartRequest(_messages.Message):
  r"""A CloudshellUsersEnvironmentsStartRequest object.

  Fields:
    name: Name of the resource that should be started, for example
      `users/me/environments/default` or
      `users/someone@example.com/environments/default`.
    startEnvironmentRequest: A StartEnvironmentRequest resource to be passed
      as the request body.
  """

  name = _messages.StringField(1, required=True)
  startEnvironmentRequest = _messages.MessageField('StartEnvironmentRequest', 2)


class CreatePublicKeyRequest(_messages.Message):
  r"""Request message for CreatePublicKey.

  Fields:
    key: Key that should be added to the environment.
  """

  key = _messages.MessageField('PublicKey', 1)


class Empty(_messages.Message):
  r"""A generic empty message that you can re-use to avoid defining duplicated
  empty messages in your APIs. A typical example is to use it as the request
  or the response type of an API method. For instance: service Foo { rpc
  Bar(google.protobuf.Empty) returns (google.protobuf.Empty); } The JSON
  representation for `Empty` is empty JSON object `{}`.
  """



class Environment(_messages.Message):
  r"""A Cloud Shell environment, which is defined as the combination of a
  Docker image specifying what is installed on the environment and a home
  directory containing the user's data that will remain across sessions. Each
  user has a single environment with the ID "default".

  Enums:
    SizeValueValuesEnum: Indicates the size of the backing VM running the
      environment. If set to something other than DEFAULT, it will be reverted
      to the default VM size after vm_size_expire_time.
    StateValueValuesEnum: Output only. Current execution state of this
      environment.

  Fields:
    dockerImage: Required. Full path to the Docker image used to run this
      environment, e.g. "gcr.io/dev-con/cloud-devshell:latest".
    id: Output only. The environment's identifier, unique among the user's
      environments.
    name: Output only. Full name of this resource, in the format
      `users/{owner_email}/environments/{environment_id}`. `{owner_email}` is
      the email address of the user to whom this environment belongs, and
      `{environment_id}` is the identifier of this environment. For example,
      `users/someone@example.com/environments/default`.
    publicKeys: Output only. Public keys associated with the environment.
      Clients can connect to this environment via SSH only if they possess a
      private key corresponding to at least one of these public keys. Keys can
      be added to or removed from the environment using the CreatePublicKey
      and DeletePublicKey methods.
    size: Indicates the size of the backing VM running the environment. If set
      to something other than DEFAULT, it will be reverted to the default VM
      size after vm_size_expire_time.
    sshHost: Output only. Host to which clients can connect to initiate SSH
      sessions with the environment.
    sshPort: Output only. Port to which clients can connect to initiate SSH
      sessions with the environment.
    sshUsername: Output only. Username that clients should use when initiating
      SSH sessions with the environment.
    state: Output only. Current execution state of this environment.
    vmSizeExpireTime: Output only. The time when the Environment will expire
      back to the default VM size.
    webHost: Output only. Host to which clients can connect to initiate HTTPS
      or WSS connections with the environment.
    webPorts: Output only. Ports to which clients can connect to initiate
      HTTPS or WSS connections with the environment.
  """

  class SizeValueValuesEnum(_messages.Enum):
    r"""Indicates the size of the backing VM running the environment. If set
    to something other than DEFAULT, it will be reverted to the default VM
    size after vm_size_expire_time.

    Values:
      VM_SIZE_UNSPECIFIED: The VM size is unknown.
      DEFAULT: The default VM size.
      BOOSTED: The boosted VM size.
    """
    VM_SIZE_UNSPECIFIED = 0
    DEFAULT = 1
    BOOSTED = 2

  class StateValueValuesEnum(_messages.Enum):
    r"""Output only. Current execution state of this environment.

    Values:
      STATE_UNSPECIFIED: The environment's states is unknown.
      DISABLED: The environment is not running and can't be connected to.
        Starting the environment will transition it to the STARTING state.
      STARTING: The environment is being started but is not yet ready to
        accept connections.
      RUNNING: The environment is running and ready to accept connections. It
        will automatically transition back to DISABLED after a period of
        inactivity or if another environment is started.
      DELETING: The environment is being deleted and can't be connected to.
    """
    STATE_UNSPECIFIED = 0
    DISABLED = 1
    STARTING = 2
    RUNNING = 3
    DELETING = 4

  dockerImage = _messages.StringField(1)
  id = _messages.StringField(2)
  name = _messages.StringField(3)
  publicKeys = _messages.MessageField('PublicKey', 4, repeated=True)
  size = _messages.EnumField('SizeValueValuesEnum', 5)
  sshHost = _messages.StringField(6)
  sshPort = _messages.IntegerField(7, variant=_messages.Variant.INT32)
  sshUsername = _messages.StringField(8)
  state = _messages.EnumField('StateValueValuesEnum', 9)
  vmSizeExpireTime = _messages.StringField(10)
  webHost = _messages.StringField(11)
  webPorts = _messages.IntegerField(12, repeated=True, variant=_messages.Variant.INT32)


class Operation(_messages.Message):
  r"""This resource represents a long-running operation that is the result of
  a network API call.

  Messages:
    MetadataValue: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    ResponseValue: The normal response of the operation in case of success. If
      the original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

  Fields:
    done: If the value is `false`, it means the operation is still in
      progress. If `true`, the operation is completed, and either `error` or
      `response` is available.
    error: The error result of the operation in case of failure or
      cancellation.
    metadata: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    name: The server-assigned name, which is only unique within the same
      service that originally returns it. If you use the default HTTP mapping,
      the `name` should be a resource name ending with
      `operations/{unique_id}`.
    response: The normal response of the operation in case of success. If the
      original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata associated with the operation. It typically
    contains progress information and common metadata such as create time.
    Some services might not provide such metadata. Any method that returns a
    long-running operation should document the metadata type, if any.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ResponseValue(_messages.Message):
    r"""The normal response of the operation in case of success. If the
    original method returns no data on success, such as `Delete`, the response
    is `google.protobuf.Empty`. If the original method is standard
    `Get`/`Create`/`Update`, the response should be the resource. For other
    methods, the response should have the type `XxxResponse`, where `Xxx` is
    the original method name. For example, if the original method name is
    `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

    Messages:
      AdditionalProperty: An additional property for a ResponseValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ResponseValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  done = _messages.BooleanField(1)
  error = _messages.MessageField('Status', 2)
  metadata = _messages.MessageField('MetadataValue', 3)
  name = _messages.StringField(4)
  response = _messages.MessageField('ResponseValue', 5)


class PublicKey(_messages.Message):
  r"""A public SSH key, corresponding to a private SSH key held by the client.

  Enums:
    FormatValueValuesEnum: Required. Format of this key's content.

  Fields:
    format: Required. Format of this key's content.
    key: Required. Content of this key.
    name: Output only. Full name of this resource, in the format
      `users/{owner_email}/environments/{environment_id}/publicKeys/{key_id}`.
      `{owner_email}` is the email address of the user to whom the key
      belongs. `{environment_id}` is the identifier of the environment to
      which the key grants access. `{key_id}` is the unique identifier of the
      key. For example,
      `users/someone@example.com/environments/default/publicKeys/myKey`.
  """

  class FormatValueValuesEnum(_messages.Enum):
    r"""Required. Format of this key's content.

    Values:
      FORMAT_UNSPECIFIED: Unknown format. Do not use.
      SSH_DSS: `ssh-dss` key format (see RFC4253).
      SSH_RSA: `ssh-rsa` key format (see RFC4253).
      ECDSA_SHA2_NISTP256: `ecdsa-sha2-nistp256` key format (see RFC5656).
      ECDSA_SHA2_NISTP384: `ecdsa-sha2-nistp384` key format (see RFC5656).
      ECDSA_SHA2_NISTP521: `ecdsa-sha2-nistp521` key format (see RFC5656).
    """
    FORMAT_UNSPECIFIED = 0
    SSH_DSS = 1
    SSH_RSA = 2
    ECDSA_SHA2_NISTP256 = 3
    ECDSA_SHA2_NISTP384 = 4
    ECDSA_SHA2_NISTP521 = 5

  format = _messages.EnumField('FormatValueValuesEnum', 1)
  key = _messages.BytesField(2)
  name = _messages.StringField(3)


class StandardQueryParameters(_messages.Message):
  r"""Query parameters accepted by all methods.

  Enums:
    FXgafvValueValuesEnum: V1 error format.
    AltValueValuesEnum: Data format for response.

  Fields:
    f__xgafv: V1 error format.
    access_token: OAuth access token.
    alt: Data format for response.
    callback: JSONP
    fields: Selector specifying which fields to include in a partial response.
    key: API key. Your API key identifies your project and provides you with
      API access, quota, and reports. Required unless you provide an OAuth 2.0
      token.
    oauth_token: OAuth 2.0 token for the current user.
    prettyPrint: Returns response with indentations and line breaks.
    quotaUser: Available to use for quota purposes for server-side
      applications. Can be any arbitrary string assigned to a user, but should
      not exceed 40 characters.
    trace: A tracing token of the form "token:<tokenid>" to include in api
      requests.
    uploadType: Legacy upload protocol for media (e.g. "media", "multipart").
    upload_protocol: Upload protocol for media (e.g. "raw", "multipart").
  """

  class AltValueValuesEnum(_messages.Enum):
    r"""Data format for response.

    Values:
      json: Responses with Content-Type of application/json
      media: Media download with context-dependent Content-Type
      proto: Responses with Content-Type of application/x-protobuf
    """
    json = 0
    media = 1
    proto = 2

  class FXgafvValueValuesEnum(_messages.Enum):
    r"""V1 error format.

    Values:
      _1: v1 error format
      _2: v2 error format
    """
    _1 = 0
    _2 = 1

  f__xgafv = _messages.EnumField('FXgafvValueValuesEnum', 1)
  access_token = _messages.StringField(2)
  alt = _messages.EnumField('AltValueValuesEnum', 3, default='json')
  callback = _messages.StringField(4)
  fields = _messages.StringField(5)
  key = _messages.StringField(6)
  oauth_token = _messages.StringField(7)
  prettyPrint = _messages.BooleanField(8, default=True)
  quotaUser = _messages.StringField(9)
  trace = _messages.StringField(10)
  uploadType = _messages.StringField(11)
  upload_protocol = _messages.StringField(12)


class StartEnvironmentMetadata(_messages.Message):
  r"""Message included in the metadata field of operations returned from
  StartEnvironment.

  Enums:
    StateValueValuesEnum: Current state of the environment being started.

  Fields:
    state: Current state of the environment being started.
  """

  class StateValueValuesEnum(_messages.Enum):
    r"""Current state of the environment being started.

    Values:
      STATE_UNSPECIFIED: The environment's start state is unknown.
      STARTING: The environment is in the process of being started, but no
        additional details are available.
      UNARCHIVING_DISK: Startup is waiting for the user's disk to be
        unarchived. This can happen when the user returns to Cloud Shell after
        not having used it for a while, and suggests that startup will take
        longer than normal.
      AWAITING_VM: Startup is waiting for a VM to be assigned to the
        environment. This should normally happen very quickly, but an
        environment might stay in this state for an extended period of time if
        the system is experiencing heavy load.
      AWAITING_COMPUTE_RESOURCES: Startup is waiting for compute resources to
        be assigned to the environment. This should normally happen very
        quickly, but an environment might stay in this state for an extended
        period of time if the system is experiencing heavy load.
      FINISHED: Startup has completed. If the start operation was successful,
        the user should be able to establish an SSH connection to their
        environment. Otherwise, the operation will contain details of the
        failure.
    """
    STATE_UNSPECIFIED = 0
    STARTING = 1
    UNARCHIVING_DISK = 2
    AWAITING_VM = 3
    AWAITING_COMPUTE_RESOURCES = 4
    FINISHED = 5

  state = _messages.EnumField('StateValueValuesEnum', 1)


class StartEnvironmentRequest(_messages.Message):
  r"""Request message for StartEnvironment.

  Fields:
    accessToken: The initial access token passed to the environment. If this
      is present and valid, the environment will be pre-authenticated with
      gcloud so that the user can run gcloud commands in Cloud Shell without
      having to log in. This code can be updated later by calling
      AuthorizeEnvironment.
    publicKeys: Public keys that should be added to the environment before it
      is started.
  """

  accessToken = _messages.StringField(1)
  publicKeys = _messages.MessageField('PublicKey', 2, repeated=True)


class StartEnvironmentResponse(_messages.Message):
  r"""Message included in the response field of operations returned from
  StartEnvironment once the operation is complete.

  Fields:
    environment: Environment that was started.
  """

  environment = _messages.MessageField('Environment', 1)


class Status(_messages.Message):
  r"""The `Status` type defines a logical error model that is suitable for
  different programming environments, including REST APIs and RPC APIs. It is
  used by [gRPC](https://github.com/grpc). Each `Status` message contains
  three pieces of data: error code, error message, and error details. You can
  find out more about this error model and how to work with it in the [API
  Design Guide](https://cloud.google.com/apis/design/errors).

  Messages:
    DetailsValueListEntry: A DetailsValueListEntry object.

  Fields:
    code: The status code, which should be an enum value of google.rpc.Code.
    details: A list of messages that carry the error details. There is a
      common set of message types for APIs to use.
    message: A developer-facing error message, which should be in English. Any
      user-facing error message should be localized and sent in the
      google.rpc.Status.details field, or localized by the client.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class DetailsValueListEntry(_messages.Message):
    r"""A DetailsValueListEntry object.

    Messages:
      AdditionalProperty: An additional property for a DetailsValueListEntry
        object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a DetailsValueListEntry object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  code = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  details = _messages.MessageField('DetailsValueListEntry', 2, repeated=True)
  message = _messages.StringField(3)


encoding.AddCustomJsonFieldMapping(
    StandardQueryParameters, 'f__xgafv', '$.xgafv')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_1', '1')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_2', '2')



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Module for user service account mapping API support.

Typical usage (update command):

  # When defining arguments
  user_sa_mapping_util.AddUpdateUserSaMappingFlags(parser)

  # When running the command
  user_sa_mapping_diff = user_sa_mapping_util.Diff.FromUpdateArgs(args)
  if user_sa_mapping_diff.HasUpdates():
    orig_resource = Get(...)  # to prevent unnecessary Get calls
    user_sa_mapping_update = user_sa_mapping_diff.Apply(user_sa_mapping_cls,
    orig_resource.user_sa_mapping)
    if user_sa_mapping_update.needs_update:
      new_resource.user_sa_mapping = user_sa_mapping_update.user_sa_mapping
      field_mask.append('user_sa_mapping')
  Update(..., new_resource)

  # Or alternatively, when running the command
  user_sa_mapping_update = user_sa_mapping_util.ProcessUpdateArgsLazy(
    args, user_sa_mapping_cls, lambda: Get(...).user_sa_mapping)
  if user_sa_mapping_update.needs_update:
    new_resource.user_sa_mapping = user_sa_mapping_update.user_sa_mapping
    field_mask.append('user_sa_mapping')
  Update(..., new_resource)
"""

from googlecloudsdk.calliope import arg_parsers
import six


def AddUpdateUserSaMappingFlags(parser):
  """Adds update command user service account mapping flags to an argparse parser.

  Args:
    parser: The argparse parser to add the flags to.
  """
  secure_multi_tenancy_group = parser.add_group(mutex=True)
  add_and_remove_user_mapping_group = secure_multi_tenancy_group.add_group()
  add_and_remove_user_mapping_group.add_argument(
      '--add-user-mappings',
      metavar='KEY=VALUE',
      type=arg_parsers.ArgDict(),
      action=arg_parsers.UpdateAction,
      help="""\
                List of user-to-service-account mappings to add to current mappings.
                If a mapping exists, its value is modified; otherwise, the new
                mapping is added.
            """,
  )
  add_and_remove_user_mapping_group.add_argument(
      '--remove-user-mappings',
      metavar='KEY',
      type=arg_parsers.ArgList(),
      action=arg_parsers.UpdateAction,
      help="""\
                List of user-to-service-account mappings to remove from the
                current mappings. If a mapping does not exist, it is ignored.
        """,
  )
  secure_multi_tenancy_group.add_argument(
      '--identity-config-file',
      help="""\
                Path to a YAML (or JSON) file that contains the configuration for [Secure Multi-Tenancy](/dataproc/docs/concepts/iam/sa-multi-tenancy)
                on the cluster. The path can be a Cloud Storage URL (example: 'gs://path/to/file')
                or a local filesystem path. The mappings provided in the file will overwrite existing mappings.

                The YAML file is formatted as follows:

                ```
                  # Mapping header (first line) required.
                  user_service_account_mapping:
                    bob@company.com: service-account-bob@project.iam.gserviceaccount.com
                    alice@company.com: service-account-alice@project.iam.gserviceaccount.com
                ```
            """,
  )


def GetAddUserSaMappingDictFromArgs(args):
  """Returns the add user mapping dict from the parsed args.

  Args:
    args: The parsed args.

  Returns:
    The add user mapping dict from the parsed args.
  """
  return args.add_user_mappings


def GetRemoveUserSaMappingListFromArgs(args):
  """Returns the remove user mapping list from the parsed args.

  Args:
    args: The parsed args.

  Returns:
    The remove user mapping list from the parsed args.
  """
  return args.remove_user_mappings


class UpdateResult(object):
  """Result type for Diff application.

  Attributes:
    needs_update: bool, whether the diff resulted in any changes to the existing
      user service account mapping proto.
    _user_sa_mapping: UserServiceAccountMappingValue, the new populated
      UserServiceAccountMappingValue object. If needs_update is False, this is
      identical to the original UserServiceAccountMappingValue object.
  """

  def __init__(self, needs_update, user_sa_mapping):
    self.needs_update = needs_update
    self._user_sa_mapping = user_sa_mapping

  @property
  def user_sa_mapping(self):
    """Returns the new user service account mapping.

    Raises:
      ValueError: if needs_update is False.
    """
    if not self.needs_update:
      raise ValueError(
          'If no update is needed (self.needs_update == False), '
          'checking user service account mapping is unnecessary.'
      )
    return self._user_sa_mapping

  def GetOrNone(self):
    """Returns the new user service account mapping if an update is needed or None otherwise.

    NOTE: If this function returns None, make sure not to include the user
    service account mapping field in the field mask of the update command.
    Otherwise, it's possible to inadvertently clear the user service account
    mapping on the resource.
    """
    try:
      return self.user_sa_mapping
    except ValueError:
      return None


class Diff(object):
  """Class for diffing user service account mapping."""

  def __init__(self, add_user_mapping=None, remove_user_mapping=None):
    """Initialize a Diff.

    Args:
      add_user_mapping: {str: str}, any user-to-service-account mapping to be
        added
      remove_user_mapping: List[str], any user-to-service-account mappings to be
        removed

    Returns:
      Diff.
    """
    self._add_user_mapping = add_user_mapping
    self._remove_user_mapping = remove_user_mapping

  def _AddUserSaMapping(self, new_user_sa_mapping):
    new_user_sa_mapping = new_user_sa_mapping.copy()
    new_user_sa_mapping.update(self._add_user_mapping)
    return new_user_sa_mapping

  def _RemoveUserSaMapping(self, new_user_sa_mapping):
    new_user_sa_mapping = new_user_sa_mapping.copy()
    for key in self._remove_user_mapping:
      new_user_sa_mapping.pop(key, None)
    return new_user_sa_mapping

  def _GetExistingUserSaMappingDict(self, user_sa_mapping):
    if not user_sa_mapping:
      return {}
    return {l.key: l.value for l in user_sa_mapping.additionalProperties}

  def _PackageUserSaMapping(self, user_sa_mapping_cls, user_sa_mapping):
    """Converts a dictionary representing a user service account mapping into an instance of a specified class.

    Args:
      user_sa_mapping_cls: The class to instantiate, which should have an
        `AdditionalProperty` inner class and an `additionalProperties`
        attribute.
      user_sa_mapping: A dictionary where keys are user identifiers and values
        are service account identifiers.

    Returns:
      An instance of `user_sa_mapping_cls` populated with the key-value pairs
      from `user_sa_mapping`.
    """
    # Sorted for test stability
    return user_sa_mapping_cls(
        additionalProperties=[
            user_sa_mapping_cls.AdditionalProperty(key=key, value=value)
            for key, value in sorted(six.iteritems(user_sa_mapping))
        ]
    )

  def HasUpdates(self):
    """Returns true iff the diff is non-empty which means user service account mapping has been updated."""
    return any([self._add_user_mapping, self._remove_user_mapping])

  def Apply(self, user_sa_mapping_cls, existing_user_sa_mapping=None):
    """Apply this Diff to the existing user service account mapping.

    Args:
      user_sa_mapping_cls: type, the UserServiceAccountMappingValue class for
        the resource.
      existing_user_sa_mapping: UserServiceAccountMappingValue, the existing
        UserServiceAccountMappingValue object for the original resource (or
        None, which is treated the same as empty user service account mapping)

    Returns:
      UpdateResult, the result of applying the diff.
    """
    existing_user_sa_mapping_dict = self._GetExistingUserSaMappingDict(
        existing_user_sa_mapping
    )
    new_user_sa_mapping_dict = existing_user_sa_mapping_dict.copy()

    if self._add_user_mapping:
      new_user_sa_mapping_dict = self._AddUserSaMapping(
          new_user_sa_mapping_dict
      )
    if self._remove_user_mapping:
      new_user_sa_mapping_dict = self._RemoveUserSaMapping(
          new_user_sa_mapping_dict
      )

    needs_update = new_user_sa_mapping_dict != existing_user_sa_mapping_dict
    return UpdateResult(
        needs_update,
        self._PackageUserSaMapping(
            user_sa_mapping_cls, new_user_sa_mapping_dict
        ),
    )

  @classmethod
  def FromUpdateArgs(cls, args):
    return cls(args.add_user_mappings, args.remove_user_mappings)


def ProcessUpdateArgsLazy(
    args, user_sa_mapping_cls, orig_user_sa_mapping_thunk
):
  """Returns the result of applying the diff constructed from args.

  Lazily fetches the original user service account mapping value if needed.

  Args:
    args: argparse.Namespace, the parsed arguments with add_user_mapping and
      remove_user_mapping
    user_sa_mapping_cls: type, the UserSaMappingValue class for the new user
      service account mapping.
    orig_user_sa_mapping_thunk: callable, a thunk which will return the original
      user_service_account_mapping object when evaluated.

  Returns:
    UpdateResult: the result of applying the diff.
  """
  diff = Diff.FromUpdateArgs(args)
  orig_user_sa_mapping = (
      orig_user_sa_mapping_thunk() if diff.HasUpdates() else None
  )
  return diff.Apply(user_sa_mapping_cls, orig_user_sa_mapping)



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""API library for Supported Services."""


from apitools.base.py import list_pager
from googlecloudsdk.api_lib.accesscontextmanager import util


class Client(object):
  """High-level API client for Supported Services."""

  def __init__(self, client=None, messages=None, version='v1'):
    self.client = client or util.GetClient(version=version)
    self.messages = messages or self.client.MESSAGES_MODULE

  def Get(self, supported_services_ref):
    return self.client.services.Get(
        self.messages.AccesscontextmanagerServicesGetRequest(
            name=supported_services_ref.RelativeName()
        )
    )

  def List(self, page_size=200, limit=None):
    """Make API call to list VPC Service Controls supported services.

    Args:
      page_size: The page size to list.
      limit: The maximum number of services to display.

    Returns:
      The list of VPC Service Controls supported services
    """
    req = self.messages.AccesscontextmanagerServicesListRequest()
    return list_pager.YieldFromList(
        self.client.services,
        req,
        limit=limit,
        batch_size_attribute='pageSize',
        batch_size=page_size,
        field='supportedServices',
    )



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Spanner instance partition API helper."""


import datetime

from apitools.base.py import list_pager
from cloudsdk.google.protobuf import timestamp_pb2
from googlecloudsdk.api_lib.spanner import response_util
from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.core import properties
from googlecloudsdk.core import resources


# The list of pre-defined IAM roles in Spanner.
KNOWN_ROLES = [
    'roles/spanner.admin',
    'roles/spanner.databaseAdmin',
    'roles/spanner.databaseReader',
    'roles/spanner.databaseUser',
    'roles/spanner.viewer',
]

# Timeout to use in ListInstancePartitions for unreachable instance partitions.
UNREACHABLE_INSTANCE_PARTITION_TIMEOUT = datetime.timedelta(seconds=20)

_API_NAME = 'spanner'
_API_VERSION = 'v1'


def Create(
    instance_ref,
    instance_partition,
    config,
    description,
    nodes,
    processing_units=None,
    autoscaling_min_nodes=None,
    autoscaling_max_nodes=None,
    autoscaling_min_processing_units=None,
    autoscaling_max_processing_units=None,
    autoscaling_high_priority_cpu_target=None,
    autoscaling_total_cpu_target=None,
    autoscaling_storage_target=None,
):
  """Create a new instance partition."""
  client = apis.GetClientInstance(_API_NAME, _API_VERSION)
  # Module containing the definitions of messages for the specified API.
  msgs = apis.GetMessagesModule(_API_NAME, _API_VERSION)
  config_ref = resources.REGISTRY.Parse(
      config,
      params={'projectsId': properties.VALUES.core.project.GetOrFail},
      collection='spanner.projects.instanceConfigs',
  )
  instance_partition_obj = msgs.InstancePartition(
      config=config_ref.RelativeName(), displayName=description
  )
  if nodes:
    instance_partition_obj.nodeCount = nodes
  elif processing_units:
    instance_partition_obj.processingUnits = processing_units
  elif (
      autoscaling_min_nodes
      or autoscaling_max_nodes
      or autoscaling_min_processing_units
      or autoscaling_max_processing_units
      or autoscaling_high_priority_cpu_target is not None
      or autoscaling_total_cpu_target is not None
      or autoscaling_storage_target
  ):
    instance_partition_obj.autoscalingConfig = msgs.AutoscalingConfig(
        autoscalingLimits=msgs.AutoscalingLimits(
            minNodes=autoscaling_min_nodes,
            maxNodes=autoscaling_max_nodes,
            minProcessingUnits=autoscaling_min_processing_units,
            maxProcessingUnits=autoscaling_max_processing_units,
        ),
        autoscalingTargets=msgs.AutoscalingTargets(
            highPriorityCpuUtilizationPercent=autoscaling_high_priority_cpu_target,
            totalCpuUtilizationPercent=autoscaling_total_cpu_target,
            storageUtilizationPercent=autoscaling_storage_target,
        ),
    )
  req = msgs.SpannerProjectsInstancesInstancePartitionsCreateRequest(
      parent=instance_ref.RelativeName(),
      createInstancePartitionRequest=msgs.CreateInstancePartitionRequest(
          instancePartitionId=instance_partition,
          instancePartition=instance_partition_obj,
      ),
  )
  return client.projects_instances_instancePartitions.Create(req)


def Get(instance_partition_ref):
  """Get an instance partition by name."""
  client = apis.GetClientInstance(_API_NAME, _API_VERSION)
  msgs = apis.GetMessagesModule(_API_NAME, _API_VERSION)
  req = msgs.SpannerProjectsInstancesInstancePartitionsGetRequest(
      name=instance_partition_ref.RelativeName()
  )
  return client.projects_instances_instancePartitions.Get(req)


def Patch(
    instance_partition_ref,
    description=None,
    nodes=None,
    processing_units=None,
    autoscaling_min_nodes=None,
    autoscaling_max_nodes=None,
    autoscaling_min_processing_units=None,
    autoscaling_max_processing_units=None,
    autoscaling_high_priority_cpu_target=None,
    autoscaling_total_cpu_target=None,
    autoscaling_storage_target=None,
):
  """Update an instance partition."""
  fields = []
  if description is not None:
    fields.append('displayName')
  if nodes is not None:
    fields.append('nodeCount')
  if processing_units is not None:
    fields.append('processingUnits')

  if (
      (autoscaling_min_nodes and autoscaling_max_nodes)
      or (autoscaling_min_processing_units and autoscaling_max_processing_units)
  ) and (
      (autoscaling_high_priority_cpu_target is not None or
       autoscaling_total_cpu_target is not None)
      and autoscaling_storage_target
  ):
    fields.append('autoscalingConfig')
  else:
    if autoscaling_min_nodes:
      fields.append('autoscalingConfig.autoscalingLimits.minNodes')
    if autoscaling_max_nodes:
      fields.append('autoscalingConfig.autoscalingLimits.maxNodes')
    if autoscaling_min_processing_units:
      fields.append('autoscalingConfig.autoscalingLimits.minProcessingUnits')
    if autoscaling_max_processing_units:
      fields.append('autoscalingConfig.autoscalingLimits.maxProcessingUnits')
    if autoscaling_high_priority_cpu_target is not None:
      fields.append(
          'autoscalingConfig.autoscalingTargets.highPriorityCpuUtilizationPercent'
      )
    if autoscaling_total_cpu_target is not None:
      fields.append(
          'autoscalingConfig.autoscalingTargets.totalCpuUtilizationPercent'
      )
    if autoscaling_storage_target:
      fields.append(
          'autoscalingConfig.autoscalingTargets.storageUtilizationPercent'
      )

  client = apis.GetClientInstance(_API_NAME, _API_VERSION)
  msgs = apis.GetMessagesModule(_API_NAME, _API_VERSION)

  instance_partition_obj = msgs.InstancePartition(displayName=description)
  if processing_units:
    instance_partition_obj.processingUnits = processing_units
  elif nodes:
    instance_partition_obj.nodeCount = nodes
  elif (
      autoscaling_min_nodes
      or autoscaling_max_nodes
      or autoscaling_min_processing_units
      or autoscaling_max_processing_units
      or autoscaling_high_priority_cpu_target is not None
      or autoscaling_total_cpu_target is not None
      or autoscaling_storage_target
  ):
    instance_partition_obj.autoscalingConfig = msgs.AutoscalingConfig(
        autoscalingLimits=msgs.AutoscalingLimits(
            minNodes=autoscaling_min_nodes,
            maxNodes=autoscaling_max_nodes,
            minProcessingUnits=autoscaling_min_processing_units,
            maxProcessingUnits=autoscaling_max_processing_units,
        ),
        autoscalingTargets=msgs.AutoscalingTargets(
            highPriorityCpuUtilizationPercent=autoscaling_high_priority_cpu_target,
            totalCpuUtilizationPercent=autoscaling_total_cpu_target,
            storageUtilizationPercent=autoscaling_storage_target,
        ),
    )

  req = msgs.SpannerProjectsInstancesInstancePartitionsPatchRequest(
      name=instance_partition_ref.RelativeName(),
      updateInstancePartitionRequest=msgs.UpdateInstancePartitionRequest(
          fieldMask=','.join(fields), instancePartition=instance_partition_obj
      ),
  )
  return client.projects_instances_instancePartitions.Patch(req)


def List(instance_ref):
  """List instance partitions in the project."""
  client = apis.GetClientInstance(_API_NAME, _API_VERSION)
  msgs = apis.GetMessagesModule(_API_NAME, _API_VERSION)
  tp_proto = timestamp_pb2.Timestamp()
  tp_proto.FromDatetime(
      datetime.datetime.now(tz=datetime.timezone.utc)
      + UNREACHABLE_INSTANCE_PARTITION_TIMEOUT
  )
  req = msgs.SpannerProjectsInstancesInstancePartitionsListRequest(
      parent=instance_ref.RelativeName(),
      instancePartitionDeadline=tp_proto.ToJsonString(),
  )
  return list_pager.YieldFromList(
      client.projects_instances_instancePartitions,
      req,
      field='instancePartitions',
      batch_size_attribute='pageSize',
      get_field_func=response_util.GetFieldAndLogUnreachableInstancePartitions,
  )


def Delete(instance_partition_ref):
  """Delete an instance partition."""
  client = apis.GetClientInstance(_API_NAME, _API_VERSION)
  msgs = apis.GetMessagesModule(_API_NAME, _API_VERSION)
  req = msgs.SpannerProjectsInstancesInstancePartitionsDeleteRequest(
      name=instance_partition_ref.RelativeName()
  )
  return client.projects_instances_instancePartitions.Delete(req)



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Vertex AI endpoints stream-raw-predict command."""


import json
import sys

from googlecloudsdk.api_lib.ai.endpoints import client
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.ai import constants
from googlecloudsdk.command_lib.ai import endpoint_util
from googlecloudsdk.command_lib.ai import flags
from googlecloudsdk.command_lib.ai import region_util
from googlecloudsdk.core import exceptions as core_exceptions
from googlecloudsdk.core.console import console_io
import six


def _AddArgs(parser):
  flags.AddEndpointResourceArg(
      parser,
      'to do online stream raw prediction',
      prompt_func=region_util.PromptForOpRegion,
  )

  flags.GetRawPredictHeadersArg().AddToParser(parser)
  flags.GetRawPredictRequestArg().AddToParser(parser)


def _Run(args, version):
  """Run Vertex AI online prediction."""
  endpoint_ref = args.CONCEPTS.endpoint.Parse()
  args.region = endpoint_ref.AsDict()['locationsId']

  with endpoint_util.AiplatformEndpointOverrides(version, region=args.region):
    if args.request.startswith('@'):
      request = console_io.ReadFromFileOrStdin(args.request[1:], binary=True)
    else:
      request = args.request.encode('utf-8')

    endpoints_client = client.EndpointsClient(version=version)
    for response in endpoints_client.StreamRawPredict(
        endpoint_ref, args.http_headers, request
    ):
      # Workaround since gcloud only supports protobufs as JSON objects. Since
      # stream raw predict can return anything, write stream raw bytes to
      # stdout.
      if not args.IsSpecified('format'):
        sys.stdout.buffer.write(response)
        continue

      # If user asked for formatting, assume it's a JSON object.
      try:
        yield json.loads(response.decode('utf-8'))
      except ValueError:
        raise core_exceptions.Error(
            'No JSON object could be decoded from the HTTP response body:\n'
            + six.text_type(response)
        )


@base.UniverseCompatible
@base.ReleaseTracks(base.ReleaseTrack.GA)
class StreamRawPredict(base.Command):
  """Run Vertex AI online stream raw prediction.

  `{command}` sends a stream raw prediction request to a Vertex AI endpoint. The
  request can be given on the command line or read from a file or stdin.

  ## EXAMPLES

  To predict against an endpoint ``123'' under project ``example'' in region
  ``us-central1'', reading the request from the command line, run:

    $ {command} 123 --project=example --region=us-central1 --request='{
        "instances": [
          { "values": [1, 2, 3, 4], "key": 1 },
          { "values": [5, 6, 7, 8], "key": 2 }
        ]
      }'

  If the request body was in the file ``input.json'', run:

    $ {command} 123 --project=example --region=us-central1 --request=@input.json

  To send the image file ``image.jpeg'' and set the *content type*, run:

    $ {command} 123 --project=example --region=us-central1
    --http-headers=Content-Type=image/jpeg --request=@image.jpeg
  """

  @staticmethod
  def Args(parser):
    _AddArgs(parser)

  def Run(self, args):
    return _Run(args, constants.GA_VERSION)


@base.UniverseCompatible
@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.ALPHA)
class StreamRawPredictBeta(StreamRawPredict):
  """Run Vertex AI online stream raw prediction.

  `{command}` sends a stream raw prediction request to a Vertex AI endpoint. The
  request can be given on the command line or read from a file or stdin.

  ## EXAMPLES

  To predict against an endpoint ``123'' under project ``example'' in region
  ``us-central1'', reading the request from the command line, run:

    $ {command} 123 --project=example --region=us-central1 --request='{
        "instances": [
          { "values": [1, 2, 3, 4], "key": 1 },
          { "values": [5, 6, 7, 8], "key": 2 }
        ]
      }'

  If the request body was in the file ``input.json'', run:

    $ {command} 123 --project=example --region=us-central1 --request=@input.json

  To send the image file ``image.jpeg'' and set the *content type*, run:

    $ {command} 123 --project=example --region=us-central1
    --http-headers=Content-Type=image/jpeg --request=@image.jpeg
  """

  def Run(self, args):
    return _Run(args, constants.BETA_VERSION)



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command group for Container Registry to Artifact Registry upgrade."""


from googlecloudsdk.calliope import base


@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.GA)
class Upgrade(base.Group):
  r"""Commands to support Container Registry to Artifact Registry upgrade.

  To print an equivalent Artifact Registry IAM policy for 'gcr.io/my-project':

      $ {command} print-iam-policy gcr.io --project=my-project

  To migrate a project from Container Registry to Artifact Registry using gcr.io
  repos:

      $ {command} migrate --projects=my-project

  To migrate a project from Container Registry to Artifact Registry using
  pkg.dev repos:

      $ {command} migrate --from-gcr-io=gcr.io/from-project \
        --to-pkg-dev=to-project/to-repo
  """

  category = base.CI_CD_CATEGORY



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Implements command to list ongoing and completed patch jobs."""


from apitools.base.py import list_pager
from googlecloudsdk.api_lib.compute.os_config import utils as osconfig_api_utils
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute.os_config import utils as osconfig_command_utils
from googlecloudsdk.core import properties


_DEFAULT_LIMIT = 10


def _TransformPatchJobDisplayName(resource):
  """Returns the display name of a patch job."""

  max_len = 15  # Show only the first 15 characters if display name is long.

  if resource.get('displayName', ''):
    name = resource['displayName']
  elif resource.get('patchDeployment', ''):
    name = osconfig_command_utils.GetResourceName(resource['patchDeployment'])
  else:
    name = ''

  return (name[:max_len] + '..') if len(name) > max_len else name


def _TransformPatchJobDescription(resource):
  max_len = 30  # Show only the first 30 characters if description is long.
  description = resource.get('description', '')
  return (description[:max_len] +
          '..') if len(description) > max_len else description


def _TransformState(resource):
  state = resource.get('state', '')

  if 'instanceDetailsSummary' in resource:
    num_instances_pending_reboot = int(resource['instanceDetailsSummary'].get(
        'instancesSucceededRebootRequired', '0'))
    if state == 'SUCCEEDED' and num_instances_pending_reboot > 0:
      return 'SUCCEEDED_INSTANCES_PENDING_REBOOT'

  return state


def _TransformNumInstances(resource):
  """Sums up number of instances in a patch job."""
  if 'instanceDetailsSummary' not in resource:
    return None

  instance_details_summary = resource['instanceDetailsSummary']
  num_instances = 0
  for status in instance_details_summary:
    num_instances += int(instance_details_summary.get(status, 0))
  return num_instances


def _MakeGetUriFunc(registry):
  """Returns a transformation function from a patch job resource to an URI."""

  def UriFunc(resource):
    ref = registry.Parse(
        resource.name,
        params={
            'projects': properties.VALUES.core.project.GetOrFail,
            'patchJobs': resource.name
        },
        collection='osconfig.projects.patchJobs')
    return ref.SelfLink()

  return UriFunc


def _Args(parser, release_track):
  """Parses input flags and sets up output formats."""

  base.LIMIT_FLAG.SetDefault(parser, _DEFAULT_LIMIT)
  parser.display_info.AddFormat("""
        table(
          name.basename():label=ID,
          display_name():label=NAME,
          description(),
          create_time,
          update_time,
          state(),
          targeted_instances()
        )
      """)
  parser.display_info.AddTransforms({
      'display_name': _TransformPatchJobDisplayName,
      'description': _TransformPatchJobDescription,
      'state': _TransformState,
      'targeted_instances': _TransformNumInstances,
  })
  registry = osconfig_api_utils.GetRegistry(release_track)
  parser.display_info.AddUriFunc(_MakeGetUriFunc(registry))


@base.ReleaseTracks(base.ReleaseTrack.GA)
class List(base.ListCommand):
  """List ongoing and completed patch jobs.

  ## EXAMPLES

  To list patch jobs in the current project, run:

        $ {command}

  """

  @staticmethod
  def Args(parser):
    _Args(parser, base.ReleaseTrack.GA)

  def Run(self, args):
    project = properties.VALUES.core.project.GetOrFail()

    release_track = self.ReleaseTrack()
    client = osconfig_api_utils.GetClientInstance(release_track)
    messages = osconfig_api_utils.GetClientMessages(release_track)

    request = messages.OsconfigProjectsPatchJobsListRequest(
        pageSize=args.page_size,
        parent=osconfig_command_utils.GetProjectUriPath(project))

    return list_pager.YieldFromList(
        client.projects_patchJobs,
        request,
        limit=args.limit,
        batch_size=osconfig_command_utils.GetListBatchSize(args),
        field='patchJobs',
        batch_size_attribute='pageSize',
    )


@base.ReleaseTracks(base.ReleaseTrack.BETA)
class ListBeta(List):
  """List ongoing and completed patch jobs.

  ## EXAMPLES

  To list patch jobs in the current project, run:

        $ {command}

  """

  @staticmethod
  def Args(parser):
    _Args(parser, base.ReleaseTrack.BETA)



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Resource definitions for Cloud Platform Apis generated from apitools."""

import enum


BASE_URL = 'https://bigquerydatatransfer.googleapis.com/v1/'
DOCS_URL = 'https://cloud.google.com/bigquery-transfer/'


class Collections(enum.Enum):
  """Collections for all supported apis."""

  PROJECTS = (
      'projects',
      'projects/{projectsId}',
      {},
      ['projectsId'],
      True
  )
  PROJECTS_DATASOURCES = (
      'projects.dataSources',
      '{+name}',
      {
          '':
              'projects/{projectsId}/dataSources/{dataSourcesId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS = (
      'projects.locations',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_DATASOURCES = (
      'projects.locations.dataSources',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/dataSources/'
              '{dataSourcesId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_TRANSFERCONFIGS = (
      'projects.locations.transferConfigs',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/transferConfigs/'
              '{transferConfigsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_TRANSFERCONFIGS_RUNS = (
      'projects.locations.transferConfigs.runs',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/transferConfigs/'
              '{transferConfigsId}/runs/{runsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_TRANSFERCONFIGS_TRANSFERRESOURCES = (
      'projects.locations.transferConfigs.transferResources',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/transferConfigs/'
              '{transferConfigsId}/transferResources/{transferResourcesId}',
      },
      ['name'],
      True
  )
  PROJECTS_TRANSFERCONFIGS = (
      'projects.transferConfigs',
      '{+name}',
      {
          '':
              'projects/{projectsId}/transferConfigs/{transferConfigsId}',
      },
      ['name'],
      True
  )
  PROJECTS_TRANSFERCONFIGS_RUNS = (
      'projects.transferConfigs.runs',
      '{+name}',
      {
          '':
              'projects/{projectsId}/transferConfigs/{transferConfigsId}/'
              'runs/{runsId}',
      },
      ['name'],
      True
  )
  PROJECTS_TRANSFERCONFIGS_TRANSFERRESOURCES = (
      'projects.transferConfigs.transferResources',
      '{+name}',
      {
          '':
              'projects/{projectsId}/transferConfigs/{transferConfigsId}/'
              'transferResources/{transferResourcesId}',
      },
      ['name'],
      True
  )

  def __init__(self, collection_name, path, flat_paths, params,
               enable_uri_parsing):
    self.collection_name = collection_name
    self.path = path
    self.flat_paths = flat_paths
    self.params = params
    self.enable_uri_parsing = enable_uri_parsing



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Base classes for commands for MembershipFeature resource."""


from apitools.base.py import exceptions as apitools_exceptions
from googlecloudsdk.calliope import base as calliope_base
from googlecloudsdk.command_lib.container.fleet import base as hub_base
from googlecloudsdk.command_lib.container.fleet.features import info


class MembershipFeatureCommand(hub_base.HubCommand):
  """MembershipFeatureCommand is a mixin adding common utils to the MembershipFeature commands."""
  mf_name = ''  # Derived commands should set this to their MembershipFeature.

  @property
  def feature(self):
    """The Feature info entry for this command's Feature."""
    return info.Get(self.mf_name)

  def MembershipFeatureResourceName(self, membership_path):
    """Builds the full MembershipFeature name, using the membership path."""
    return f'{membership_path}/features/{self.mf_name}'

  def GetMembershipFeature(self, membership_path):
    """Fetch this command's MembershipFeature from the API."""
    return self.hubclient_v2.GetMembershipFeature(
        self.MembershipFeatureResourceName(membership_path)
    )


class UpdateCommandMixin(MembershipFeatureCommand):
  """A mixin for functionality to update a MembershipFeature."""

  def UpdateV2(self, membership_path, mask, patch):
    membershipfeature_path = self.MembershipFeatureResourceName(membership_path)
    op = self.hubclient_v2.UpdateMembershipFeature(
        membershipfeature_path, mask, patch
    )
    msg = (
        f'Waiting for MembershipFeature {membershipfeature_path} to be updated'
    )
    return self.WaitForHubOp(
        self.hubclient_v2.membership_feature_waiter,
        op,
        message=msg,
        warnings=False,
    )


class UpdateCommand(UpdateCommandMixin, calliope_base.UpdateCommand):
  """Base class for the command that updates a MembershipFeature."""


class DeleteCommandMixin(MembershipFeatureCommand):
  """A mixin for functionality to delte a MembershipFeature."""

  def DeleteV2(self, membership_path):
    membershipfeature_path = self.MembershipFeatureResourceName(membership_path)

    try:
      op = self.hubclient_v2.DeleteMembershipFeature(membershipfeature_path)
    except apitools_exceptions.HttpNotFoundError:
      return  # Already deleted.

    msg = (
        f'Waiting for MembershipFeature {membershipfeature_path} to be deleted'
    )
    return self.WaitForHubOp(
        self.hubclient_v2.resourceless_waiter,
        op,
        message=msg,
        warnings=False,
    )


class DeleteCommand(DeleteCommandMixin, calliope_base.DeleteCommand):
  """Base class for the command that deletes a MembershipFeature."""



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Request processor for Edge Cloud Zone surface arguments."""


from googlecloudsdk.calliope import exceptions


def SetParentForZonesList(ref, args, request):
  """Sets the parent field in the request for zone list."""
  del ref

  if args.IsSpecified('project'):
    request.parent = f'projects/{args.project}/locations/{args.location}'
  elif args.IsSpecified('organization'):
    request.parent = (
        f'organizations/{args.organization}/locations/{args.location}'
    )
  else:
    raise exceptions.OneOfArgumentsRequiredException(
        ['--organization', '--project'],
        'Error: Either --organization or --project must be specified.'
    )
  return request


def SetNameForZoneDescribe(ref, args, request):
  """Sets the name field in the request for zone describe."""
  del ref

  if args.IsSpecified('project'):
    request.name = (
        f'projects/{args.project}/locations/{args.location}/zones/{args.zone}'
    )
  elif args.IsSpecified('organization'):
    request.name = (
        f'organizations/{args.organization}/locations/{args.location}/zones/{args.zone}'
    )
  else:
    raise exceptions.OneOfArgumentsRequiredException(
        ['--organization', '--project'],
        'Error: Either --organization or --project must be specified.'
    )
  return request





# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""The `app instances describe` command."""


from googlecloudsdk.api_lib.app import appengine_api_client
from googlecloudsdk.calliope import base
from googlecloudsdk.core import properties
from googlecloudsdk.core import resources


@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.BETA)
class Describe(base.Command):
  """Display all data about an existing instance."""

  detailed_help = {
      'EXAMPLES': """\
          To show all data about instance i1 for service s1 and version v1, run:

              $ {command} --service=s1 --version=v1 i1
          """,
  }

  @staticmethod
  def Args(parser):
    parser.add_argument(
        'instance',
        help='The instance ID.')
    parser.add_argument(
        '--service', '-s', required=True,
        help='The service ID.')
    parser.add_argument(
        '--version', '-v', required=True,
        help='The version ID.')

  def Run(self, args):
    api_client = appengine_api_client.GetApiClientForTrack(self.ReleaseTrack())
    params = {'appsId': properties.VALUES.core.project.GetOrFail,
              'servicesId': args.service,
              'versionsId': args.version}
    res = resources.REGISTRY.Parse(args.instance,
                                   params=params,
                                   collection='appengine.apps.services.'
                                              'versions.instances')
    return api_client.GetInstanceResource(res)



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for removing labels from disks."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute import disks_util as api_util
from googlecloudsdk.api_lib.compute.operations import poller
from googlecloudsdk.api_lib.util import waiter
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute import flags
from googlecloudsdk.command_lib.compute import labels_doc_helper
from googlecloudsdk.command_lib.compute import labels_flags
from googlecloudsdk.command_lib.compute.disks import flags as disks_flags
from googlecloudsdk.command_lib.util.args import labels_util


class RemoveLabels(base.UpdateCommand):
  """remove-labels command for disks."""

  DISK_ARG = None

  @classmethod
  def Args(cls, parser):
    cls.DISK_ARG = disks_flags.MakeDiskArg(plural=False)
    cls.DISK_ARG.AddArgument(parser)
    labels_flags.AddArgsForRemoveLabels(parser)

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client.apitools_client
    messages = holder.client.messages
    disk_ref = self.DISK_ARG.ResolveAsResource(
        args,
        holder.resources,
        scope_lister=flags.GetDefaultScopeLister(holder.client))
    remove_labels = labels_util.GetUpdateLabelsDictFromArgs(args)
    disk_info = api_util.GetDiskInfo(disk_ref, client, messages)
    disk = disk_info.GetDiskResource()

    if args.all:
      # removing all existing labels from the disk.
      remove_labels = {}
      if disk.labels:
        for label in disk.labels.additionalProperties:
          remove_labels[label.key] = label.value
    labels_diff = labels_util.Diff(subtractions=remove_labels)

    set_label_req = disk_info.GetSetLabelsRequestMessage()
    labels_update = labels_diff.Apply(set_label_req.LabelsValue, disk.labels)
    request = disk_info.GetSetDiskLabelsRequestMessage(
        disk, labels_update.GetOrNone())
    if not labels_update.needs_update:
      return disk

    service = disk_info.GetService()
    operation = service.SetLabels(request)
    operation_ref = holder.resources.Parse(
        operation.selfLink, collection=disk_info.GetOperationCollection())

    operation_poller = poller.Poller(service)
    return waiter.WaitFor(
        operation_poller, operation_ref,
        'Updating labels of disk [{0}]'.format(
            disk_ref.Name()))


RemoveLabels.detailed_help = (
    labels_doc_helper.GenerateDetailedHelpForRemoveLabels('disk'))



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Path Utilities for storage commands."""


import copy
import random

from googlecloudsdk.command_lib.storage import storage_url
from googlecloudsdk.core import properties
from googlecloudsdk.core.util import platforms


def generate_random_int_for_path():
  """Generates random integer for making file paths not conflict."""
  # Inspired by gsutil composite object prefixes.
  return str(random.randint(1, 10**10))


def sanitize_file_resource_for_windows(resource):
  """Returns the resource with invalid characters replaced.

  The invalid characters are only replaced if the resource URL is a FileUrl
  and the platform is Windows. This is required because Cloud URLs may have
  certain characters that are not allowed in file paths on Windows.

  Args:
    resource (Resource): The resource.

  Returns:
    The resource with invalid characters replaced from the path.
  """
  if (
      (not isinstance(resource.storage_url, storage_url.FileUrl))
      or (not platforms.OperatingSystem.IsWindows())
      or (
          not properties.VALUES.storage.convert_incompatible_windows_path_characters.GetBool()
      )
  ):
    return resource

  sanitized_resource = copy.deepcopy(resource)
  sanitized_resource.storage_url.resource_name = (
      platforms.MakePathWindowsCompatible(
          sanitized_resource.storage_url.resource_name
      )
  )
  return sanitized_resource
